#!/usr/bin/env python
#coloRNA version 1.0
#Using Python 2.3.3 with Tkinter 8.4

# **********************************************************************
#
#=+coloRNA, written by Jamie LeBarron
#
#=* This program is free software; you can redistribute it and/or      *
#=* modify it under the terms of the GNU General Public License as     *
#=* published by the Free Software Foundation; either version 2 of the *
#=* License, or (at your option) any later version.                    *
#=*                                                                    *
#=* This program is distributed in the hope that it will be useful,    *
#=* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
#=* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
#=* General Public License for more details.                           *
#=*                                                                    *
#=* You should have received a copy of the GNU General Public License  *
#=* along with this program; if not, write to the                      *
#=* Free Software Foundation, Inc.,                                    *
#=* 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.      *
#=*                                                                    *
# **********************************************************************
#Many definitions are labeled either "specialized" or "general". These
#terms denote if a function is using class specific variables, other than 
#canvas ones, or if the function works by accepting arguments (general).

import string,os,sys
from Tkinter import *
from tkFileDialog import askopenfilename,asksaveasfilename
from math import sqrt 
from tkMessageBox import showwarning

class COLORNA:
   def __init__(self,master):
     
     #create window
     sidebar=Frame(master)
     self.c = Canvas(master,bg='white',width=300,height=500)
 
     #threshold variables
     self.minthresh=DoubleVar() ; self.minthresh.set(0.0)
     self.maxthresh=DoubleVar() ; self.maxthresh.set(10.0)
     self.thrshctrl=IntVar()    ; self.thrshctrl.set(1)  #use for resolution control in expert menu
     self.BcolLdGd=IntVar()     ; self.BcolLdGd.set(0)  #determines display mode 0=bcol
     self.thrshctrlkey=BooleanVar() ; self.thrshctrlkey.set(0)  #if threshholds control key range
     self.lastBcolLdGd = self.BcolLdGd.get() #previous value, so doesnt doublefire
     self.lastminthresh=self.minthresh.get()  #to tell when thresh changes, and for quick regraph
     self.lastmaxthresh=self.maxthresh.get() 
     #residue variables
     self.ressymsz=DoubleVar()  ; self.ressymsz.set(3)   #residue symbol size
     self.resletnex=StringVar() ; self.resletnex.set('') #residue letter, not existing
     self.resletsz=IntVar()     ; self.resletsz.set(5)   #residue letter size
     self.resletnum=BooleanVar(); self.resletnum.set(0)  #switch for residue letter/num, 1=let
     self.ressymex=int(0)     #residue symbol exist
     self.ressymnex=int(3)    #residue symbol not exist
     self.reslowthst=int(2)   #residues below thresh, state
     self.reshgthst=int(2)    #residues above thresh, state
     self.lastresletnum=self.resletnum.get() #previous value, to limit radiobutton to only changes
     self.xyreslist=[]  #res#,letter,x,y position of base from xy file
     self.xybarlist=[]  #1d list of list(x1 y1 x2 y2) from xy file
     self.xycblist =[]  #1d list of list(type int, x, y, diameter) from xy file
     self.Creslist=[]   #this list changes between xy or row
     self.RCreslut={}   #[resid]=index of Creslist     
     self.bondshade='#000000' #defualt bond color
     self.xletm=IntVar()  ;  self.xletm.set(3)   #create user changable 'shoves' to perfect images
     self.yletm=IntVar()  ;  self.yletm.set(-2)
     self.xsymm=IntVar()  ;  self.xsymm.set(2)
     self.ysymm=IntVar()  ;  self.ysymm.set(-2)
     self.xbndm=IntVar()  ;  self.xbndm.set(1)  
     self.ybndm=IntVar()  ;  self.ybndm.set(-1)
     
     #file name variables
     self.pdbfn=StringVar()  ; self.pdbfn.set('')
     self.xyfn=StringVar()   ; self.xyfn.set('')
     self.rbfn=StringVar()   ; self.rbfn.set('')
     self.lrfn=StringVar()   ; self.lrfn.set('')
     self.polyfn=StringVar() ; self.polyfn.set('')
     #key variables
     self.numkey=IntVar() ; self.numkey.set(5)
     self.keysz=IntVar() ; self.keysz.set(10)
     self.keysp=IntVar() ; self.keysp.set(13)
     self.keyvnum=IntVar() ; self.keyvnum.set(5)
     self.keyshape = int(1) 
     self.xmoveold = 0   #stores 'last' place key was, for mouse movement
     self.ymoveold = 0
     #rigid body variables
     self.rbgroups=[]    #[counter]=list of residues in that group
     self.rbpoints=[]    #list of polygons, with [name,color,x,y]
     self.leftline = {}         #points to "left" of residues
     self.rightline = {}        #lut of points 'right"
     self.rbregion = []         #list of polygons, with [name,color,x,y]
     self.halo=DoubleVar() ; self.halo.set(5)                 #distance for right and left lines
     self.lorp=IntVar()    ; self.lorp.set(0)                 #for realspace  colors to be lines, or filled
     self.rblsz=DoubleVar(); self.rblsz.set(4)                #currently str for realspace line width
     self.wantregion=BooleanVar(); self.wantregion.set(False)  #if possible, want polygons?
     self.xpolm=IntVar()  ;  self.xpolm.set(3)   #user changable shoves
     self.ypolm=IntVar()  ;  self.ypolm.set(-2)
     #pdb data variables
     self.colorlut = {}  #[residue number] = Bcolumn value
     self.Clut = {}    #current look up table, either Bcol or dist
     self.RClut={}     #reverse of current LUT, [float]=list of resid integers
     self.Cmax=int(10) #current max, either pdb or distance
     self.Cmin=int(0)  #current min, either pdb, distance, or something else?
     self.pdbmin=None  #lowest value from Bcolumn
     self.pdbmax=None  #largest value from Bcolumn
     self.xyzdistlut={} #contains current distance lut from pdb
     #residue display variables -- all symbols refer to list below
     self.symlist=['circle','square','diamond','triangle,up','triangle,down']
     self.swstate=['Do not display','Display','Display as other']
     #arrow variables
     self.arrowclr=DoubleVar() ; self.arrowclr.set(1) #controls arrow existance<0, arrow color(black above 1)
     self.arrowthk=DoubleVar() ; self.arrowthk.set(3.0) #arrow body thickness
     self.arrowsrt=IntVar() ; self.arrowsrt.set(2)     #pixels away to start arrow
     self.arrowlen=IntVar() ; self.arrowlen.set(30)    #distance from arrowsrt to end arrow
     self.curselres= int(0)    #current selected residue, for xyz distance
     #tick variables
     self.tickthk=IntVar() ; self.tickthk.set(2.0) #thickness of ticks
     self.ticksrt=IntVar() ; self.ticksrt.set(7)   #closest ticks get
     self.ticklen=IntVar() ; self.ticklen.set(10)  #length of ticks
     self.wanttick=BooleanVar() ; self.wanttick.set(False)
     self.tickshade="#000000"   #rgb for shade of ticks
     
     #sidebar objects
     Label(sidebar,text="OPTIONS",bd=3,fg='white',bg='black').pack(fill=X,ipadx=5,anchor=N)
     Button(sidebar,text="Load Files",bd=2,command=self.mkfilem,width=19).pack(fill=X,ipadx=2,side=TOP)
     Button(sidebar,text="Residue Options",command=self.mkresm).pack(fill=X,ipadx=3,side=TOP)
     self.currentlabel=Label(sidebar,text="Current residue: ",bd=2,anchor=W)
     self.currentlabel.pack(fill=X,ipadx=3,side=TOP)
     self.currentvalue=Label(sidebar,text="Current value: ",bd=2,anchor=W)
     self.currentvalue.pack(fill=X,ipadx=3,side=TOP)
     self.selectedlabel=Label(sidebar,text="Selected residue: ",bd=2,anchor=W)
     self.selectedlabel.pack(fill=X,ipadx=3,side=TOP)
     self.minscale=Scale(sidebar,orient='horizontal',from_=0,to=10,label='Min. Threshold.',
                         resolution=0.1,bg='white',variable=self.minthresh,
                         command=lambda x=0,y=0:self.forcethresh(x,y))
     self.minscale.bind("<ButtonRelease-1>", self.activatescale)
     self.minscale.pack(fill=X,ipadx=3,side=TOP)
     self.maxscale=Scale(sidebar,orient='horizontal',from_=0,to=10,label='Max. Threshold.',
                         resolution=0.1,bg='white',variable=self.maxthresh,
                         command=lambda x=0,y=1:self.forcethresh(x,y))
     self.maxscale.bind("<ButtonRelease-1>", self.activatescale)                    
     self.maxscale.pack(fill=X,ipadx=3,side=TOP)
     Radiobutton(sidebar,text='Residues as letters',variable=self.resletnum,
                 anchor=W,value=True,command=self.letnumtog).pack(fill=X,ipadx=3,side=TOP)
     Radiobutton(sidebar,text='Residues as symbols',variable=self.resletnum,
                 anchor=W,value=False,command=self.letnumtog).pack(fill=X,ipadx=3,side=TOP)
     Button(sidebar,text='Key Options',command=self.mkkeym).pack(fill=X,ipadx=3,side=TOP)
     Button(sidebar,text='Rigid Body &\n Tick marker Options',command=self.mkrbm).pack(fill=X,ipadx=3,side=TOP)
     Radiobutton(sidebar,text='B column display',variable=self.BcolLdGd,anchor=W,
                 value=int(0),command=self.bdisttog).pack(fill=X,ipadx=3,side=TOP)
     Radiobutton(sidebar,text='Distance display',variable=self.BcolLdGd,anchor=W,
                 value=int(1),command=self.bdisttog).pack(fill=X,ipadx=3,side=TOP)  
     Button(sidebar,text='Save screen', command=lambda x=3:self.getmorefiles(x),activebackground='#00ff00',
            background='#76ff76').pack(fill=X,ipadx=3,side=TOP)      
     Button(sidebar,text='Advanced Options',command=self.mkavm).pack(fill=X,ipadx=3,side=BOTTOM)
     sidebar.pack(side=LEFT,fill=Y)

     #canvas
     self.c.pack(expand=YES,fill=BOTH)
     self.c.bind("<Button-1>", self.OnCanvasMouseDown)
     self.c.bind("<Any-Motion>", self.OnCanvasMouseMove)
     self.c.bind("<B1-Motion>", self.OnCanvasMouseDrag)
                          
   #make file menu
   def mkfilem(self):
     #make list of filenames before asking
     before = [self.pdbfn.get(),self.xyfn.get(),self.rbfn.get(),self.lrfn.get(),self.polyfn.get()]
     
     files=Toplevel(root,width=800)      #create new window for files
     files.transient(root)     #treat new window "worse" than root
     files.title("Select Files")
     files.geometry('+%d+%d' % (root.winfo_rootx() + 200,root.winfo_rooty() + 70))
	    
     #buttons
     a=Button(files,padx=1,pady=1,text="Select pdb file",command=lambda x=1:self.getfile(x))     
     b=Button(files,padx=1,pady=1,text='Select residue xy file',command=lambda x=2:self.getfile(x))
     c=Button(files,padx=1,pady=1,text="Select rigid body(TNT) file",command=lambda x=3:self.getfile(x))
     d=Button(files,padx=1,pady=1,text='Select left/right file',command=lambda x=4:self.getfile(x))
     e=Button(files,padx=1,pady=1,text='Select colored group file',command=lambda x=5:self.getfile(x))
     #labels
     f=Label(files,bd=3,bg='white',relief=GROOVE,anchor=W,textvariable=self.pdbfn)
     g=Label(files,bd=3,bg='white',relief=GROOVE,anchor=W,textvariable=self.xyfn)
     h=Label(files,bd=3,bg='white',relief=GROOVE,anchor=W,textvariable=self.rbfn)
     i=Label(files,bd=3,bg='white',relief=GROOVE,anchor=W,textvariable=self.lrfn)
     j=Label(files,bd=3,bg='white',relief=GROOVE,anchor=W,textvariable=self.polyfn)
     #keep changes
     execute=Button(files,text="Load files",command=lambda w=files: self.filemreact(1,before,win=w),
               bg='#afffaf',activebackground='#00ff00')          
     #discard changes
     goback=Button(files,text="Ignore changes",command=lambda w=files: self.filemreact(0,before,win=w),
               bg='#ffafaf',activebackground='#ff0000')
     rl2=Button(files,text='Reload Colored Groups',activebackground='#afffaf',
               command=lambda w=files:self.refire(1,win=w)  )
     rl1=Button(files,text='Reload Left/Right',activebackground='#afffaf',
               command=lambda w=files:self.refire(0,win=w))
     
     #placement
     a.grid(row=0,column=0,sticky=W,columnspan=2)
     b.grid(row=1,column=0,sticky=W,columnspan=2)
     c.grid(row=2,column=0,sticky=W,columnspan=2)
     d.grid(row=3,column=0,sticky=W,columnspan=2)
     e.grid(row=4,column=0,sticky=W,columnspan=2)
     f.grid(row=0,column=2,sticky=W+E,columnspan=2)
     g.grid(row=1,column=2,sticky=W+E,columnspan=2)
     h.grid(row=2,column=2,sticky=W+E,columnspan=2)
     i.grid(row=3,column=2,sticky=W+E,columnspan=2)
     j.grid(row=4,column=2,sticky=W+E,columnspan=2)
     
     execute.grid(row=5,column=3,sticky=E)
     goback.grid(row=5,column=0,sticky=W)
     rl2.grid(row=5,column=2,sticky=E)
     rl1.grid(row=5,column=1,sticky=E)
     
     #only can process this window, eliminate multiple of these
     #from "Intro to Tkinter" ch10 dialog windows
     files.initial_focus = goback
     files.grab_set()
     if not files.initial_focus:
       files.initial_focus = files
     files.initial_focus.focus_set()
     return  
   
   #specialize  
   #react to file name changes
   def filemreact(self,cont,b,win=None):   
     #always close previous window
     win.destroy()

     #process changes
     if cont:
	#xy file change should be read in first, other file changes should 
	#    be caught  and regraph the changes
	if self.xyfn.get() != b[1]:
	  #read new file
	  self.xyreslist,self.xybarlist,self.xycblist = self.readmodps(self.xyfn.get())
          
	  #if there is data, graph it
	  if(self.xyreslist and self.xybarlist and self.xycblist) != []:
	     #name current res list
	     self.Creslist=self.xyreslist 
                         
             #create reverse lut [resid]= line
             #written to be back/future compatable
             self.RCreslut={}
             counter=0
             while counter < len(self.Creslist):
                self.RCreslut[self.Creslist[counter][0]]=counter
                counter=counter+1 
             
             #destroy all previous residues,and regraph
	     self.dsppositions(self.Creslist,self.resletnum.get(),which=self.ressymnex,
	             txsz=self.resletsz.get(),ssz=self.ressymsz.get())
             #destroy previous and create bonds
	     self.dspbonds(self.xybarlist,self.xycblist,self.xbndm.get(),self.ybndm.get(),color=self.bondshade)
             
             #change colors if pdb data exists
             if len(self.Clut) > int(1):
                 #self.runSC(int(0) )
                 self.changethreshold(self.BcolLdGd.get())
                 self.runSC(0)
		 self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() ) 
             
             #destroy and possibly recreate tickmarks
             self.calltickandnum()
                  
	#for pdb change
	if self.pdbfn.get() != b[0]:
	  #read data in
	  self.colorlut,self.pdbmin,self.pdbmax,self.xyzdistlut=self.readpdb(self.pdbfn.get())
          #for Bcol read data
          if self.BcolLdGd.get() == 0: self.Clut = self.colorlut
             
          #for distance
          elif self.BcolLdGd.get() == 1 and len(self.xyzdistlut) > 2:
             #use Cres to generate rist present distance
             self.bdisttog(force=1)
             
	  #read somthing?
	  if len(self.Clut) > int(1):
	     self.changethreshold(self.BcolLdGd.get())   #set thresholds to B column 
             self.runSC(0)
             
          #if residues and grpahing info, create key
          if len(self.Creslist) > 3:
             self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
             
	#for rigid body input file change
	if self.rbfn.get() != b[2] or self.xyfn.get() != b[1]:
           if self.lrfn.get() != '' and self.polyfn.get() != '' and self.xyfn.get() != '':
              self.leftline,self.rightline,self.rbgroups,self.rbpoints=self.rbcalc(self.rbfn.get(),self.Creslist,self.halo.get(),self.lrfn.get(),self.polyfn.get(),self.xyfn.get() )
          
	#for left/right file change
	if self.lrfn.get() != b[3] and self.rbfn.get() == b[2]:
           if self.rbfn.get() != '' and self.polyfn.get() != '' and self.xyfn.get() != '':
              self.leftline,self.rightline,self.rbgroups,self.rbpoints=self.rbcalc(self.rbfn.get(),self.Creslist,self.halo.get(),self.lrfn.get(),self.polyfn.get(),self.xyfn.get(),detLR=int(0) )
           
	#for colored group file
	if self.polyfn.get() != b[4] and self.rbfn.get() == b[2]: 
          self.rbpoints = self.readpolyfile(self.polyfn.get() )
         
        #only graph polygons if wanted
        if self.rbpoints != [] and self.wantregion.get() == True:
           self.graphpoly(self.rbpoints,self.lorp.get(),self.rblsz.get(),self.xpolm.get(),self.ypolm.get())

     #remove changes 
     else:
	self.pdbfn.set(b[0])  ;  self.xyfn.set(b[1])  ;  self.rbfn.set(b[2])
	self.lrfn.set(b[3])  ;  self.polyfn.set(b[4])
     return
         
   #---change self.variables that I cant change with lambda,eval,exec
   def kluge0(self,event=None): self.ressymex = int(self.r0.curselection()[0])
   def kluge1(self,event=None): self.ressymnex = int(self.r1.curselection()[0])
   def kluge5(self,event=None): self.reslowthst = int(self.r5.curselection()[0])
   def kluge6(self,event=None): self.reshgthst = int(self.r6.curselection()[0])
   def kluge2(self,event=None): self.keyshape = int(self.k0.curselection()[0])
   
   #make residue menu
   def mkresm(self):
     #list of initial values
     before=[self.ressymex,self.ressymnex,self.ressymsz.get(),self.resletnex.get(),
              self.resletsz.get(),self.reslowthst,self.reshgthst]
     
     resopt=Toplevel(root)
     resopt.transient(root)
     resopt.title("Residue Options")
     resopt.geometry('+%d+%d' % (root.winfo_rootx() + 200,root.winfo_rooty() + 70))
     
     #symbol esxist shape
     self.r0=Listbox(resopt,selectmode=BROWSE,exportselection=0,height=5)     
     for item in self.symlist:
       self.r0.insert(END,item)
     self.r0.selection_set(self.ressymex)
     self.r0.bind("<ButtonRelease-1>", self.kluge0)
     #symbol dont exist shape
     self.r1=Listbox(resopt,selectmode=BROWSE,exportselection=0,height=5)
     for item in self.symlist:
       self.r1.insert(END,item)
     self.r1.selection_set(self.ressymnex)
     self.r1.bind("<ButtonRelease-1>", self.kluge1)
     #symbol size
     self.r2=Scale(resopt,from_=0,to=30,resolution=float(0.1),orient='horizontal',
                variable=self.ressymsz)
     #letter nonexist
     self.r3=Entry(resopt,exportselection=0,textvariable=self.resletnex)
     #letter size
     self.r4=Scale(resopt,from_=0,to=30,resolution=int(1),orient='horizontal',
                variable=self.resletsz)
     #how to display residues below low thresh
     self.r5=Listbox(resopt,selectmode=BROWSE,exportselection=0,height=3)
     for item in self.swstate:
       self.r5.insert(END,item)
     self.r5.selection_set(self.reslowthst)
     self.r5.bind("<ButtonRelease-1>", self.kluge5)
     #display residues above high thresh
     self.r6=Listbox(resopt,selectmode=BROWSE,exportselection=0,height=3)
     for item in self.swstate:
       self.r6.insert(END,item)
     self.r6.selection_set(self.reshgthst)
     self.r6.bind("<ButtonRelease-1>", self.kluge6)
     
     #keep changes
     execute=Button(resopt,bg='#afffaf',activebackground='#00ff00',text="Change residues",
               command=lambda w=resopt: self.resmreact(1,before,win=w))
     #ignore changes
     goback=Button(resopt,bg='#ffafaf',activebackground='#ff0000',text="Ignore changes",
               command=lambda w=resopt: self.resmreact(0,before,win=w))
     
     #make some labels
     rl0=Label(resopt,text="Shape of existing(pdb) residue",bd=3,bg='white',anchor=W) 
     rl1=Label(resopt,text="Shape of non-existing(pdb) residue",bd=3,bg='white',anchor=W)
     rl2=Label(resopt,text="Symbol size",bd=3,bg='white',anchor=W)
     rl3=Label(resopt,text="Letter to display for non-existing(PDB) residue. Displayed normally if blank",
                 bd=3,bg='white',anchor=W)
     rl4=Label(resopt,text="Letter display size",bd=3,bg='white',anchor=W)
     rl5=Label(resopt,text="How to display residues below min. threshold",bd=3,bg='white',anchor=W)
     rl6=Label(resopt,text="How to display residues above max. threshold",bd=3,bg='white',anchor=W)
     
     #placement
     self.r0.grid(row=0,column=1)
     rl0.grid(row=0,sticky=NW+E)
     self.r1.grid(row=1,column=1)
     rl1.grid(row=1,sticky=NW+E)
     self.r2.grid(row=2,column=1,sticky=W+E)
     rl2.grid(row=2,sticky=NW+E)
     self.r3.grid(row=3,column=1)
     rl3.grid(row=3,sticky=NW+E)
     self.r4.grid(row=4,column=1,sticky=W+E)
     rl4.grid(row=4,sticky=NW+E)
     self.r5.grid(row=5,column=1)
     rl5.grid(row=5,sticky=NW+E)
     self.r6.grid(row=6,column=1)
     rl6.grid(row=6,sticky=NW+E)
     execute.grid(row=8,column=1,sticky=E)
     goback.grid(row=8,column=0,sticky=W)
     
     #ensure only one instance of this window
     resopt.initial_focus = execute
     resopt.grab_set()
     if not resopt.initial_focus:
        resopt.initial_focus = resopt
     resopt.initial_focus.focus_set()
     return
        
   #react to residue changes 
   def resmreact(self,cont,b,win=None):   
     #always close window
     win.destroy()
     
     #keep changes
     if cont:
        #for existing shape change
        if self.ressymex != b[0]:
           names=['normal','above','below',]
           self.runreshape(names)
           
        #for nonexisting shape change
        if self.ressymnex != b[1]:
           names=['notexist',]
           self.runreshape(names)
           
        #for symbol size change
        if self.ressymsz.get() != b[2]:
           self.symresize()
           
        #for nonexisting letter change
        if self.resletnex.get() != b[3]:
           #list both letters and notexist, launch 
           allletts =  self.c.find_withtag('reslet')
           dontexist = self.c.find_withtag('notexist')
           both = [n for n in dontexist if n in allletts]
           self.runCS(both)
           
        #for letter size change
        if self.resletsz.get() != b[4]:
           #get list of letters, change each of theirs font
           allletts = self.c.find_withtag('reslet')
           for one in allletts:
              self.c.itemconfigure(one,font="Helvetica "+str(self.resletsz.get())  )
              
        #for below threshold change
        if self.reslowthst != b[5]:
           #if "display as other" is involved, recalc
           if self.reslowthst == int(2) or b[5] == int(2):
              self.runSC(0)
           #just need to change color
           else: 
             self.flipthreshcolor('below',self.reslowthst)
              
        #for above threshold change
        if self.reshgthst != b[6]:
           #if "display as other" is involved, recalc
           if self.reshgthst == int(2) or b[6] == int(2):
              self.runSC(0)              
           #just need to change color
           else: 
             self.flipthreshcolor('above',self.reshgthst)
             
     #undo changes
     else:
	self.ressymex=b[0]       ; self.ressymnex=b[1]     ; self.ressymsz.set(b[2])
	self.resletnex.set(b[3]) ; self.resletsz.set(b[4]) ; self.reslowthst=b[5]
	self.reshgthst=b[6]	
     return
   
   #specific
   #create menu for key options
   def mkkeym(self):
      #initial list
      before=[self.keyshape,self.numkey.get(),self.keysz.get(),self.keysp.get(),
              self.keyvnum.get(),self.thrshctrlkey.get()]
      
      keyopt=Toplevel(root)
      keyopt.transient(root)
      keyopt.title('Key Display Options')
      keyopt.geometry('+%d+%d' % (root.winfo_rootx() + 200,root.winfo_rooty() + 70))
      
      #keyshape
      self.k0=Listbox(keyopt,selectmode=BROWSE,exportselection=0,height=5)
      self.k0.grid(row=0,column=1,sticky=NW+E)
      for item in self.symlist:
       self.k0.insert(END,item)
      self.k0.selection_set(self.keyshape)
      self.k0.bind("<ButtonRelease-1>", self.kluge2)
      Label(keyopt,text='Shape of key entries',bd=3,bg='white',
             anchor=W).grid(row=0,column=0,sticky=NW+E)
             
      #number of keys
      Scale(keyopt,orient='horizontal',from_=0,to=30,resolution=1,
            variable=self.numkey).grid(row=1,column=1,sticky=NW+E)
      Label(keyopt,text='Number of key entries',bd=3,bg='white',
            anchor=W).grid(row=1,column=0,sticky=NW+E)
            
      #key entry size
      Scale(keyopt,orient='horizontal',from_=0,to=30,resolution=1,
            variable=self.keysz).grid(row=2,column=1,sticky=NW+E)
      Label(keyopt,text='Size of key entries',bd=3,bg='white',
            anchor=W).grid(row=2,column=0,sticky=NW+E)
            
      #key spacing
      Scale(keyopt,orient='horizontal',from_=0,to=30,resolution=1,
            variable=self.keysp).grid(row=3,column=1,sticky=NW+E)
      Label(keyopt,text='Spacing of key entries',bd=3,bg='white',
            anchor=W).grid(row=3,column=0,sticky=NW+E)
            
      #key value length
      Scale(keyopt,orient='horizontal',from_=0,to=30,resolution=1,
            variable=self.keyvnum).grid(row=4,column=1,sticky=NW+E)
      Label(keyopt,text='Length of key value',bd=3,bg='white',
            anchor=W).grid(row=4,column=0,sticky=NW+E)
            
      #tie key to threshold?
      Checkbutton(keyopt,text='Use threshold for key min & max?',variable=self.thrshctrlkey,
           onvalue=True,offvalue=False,bg='white').grid(row=5,sticky=W+E,columnspan=2)
            
      #keep changes
      execute=Button(keyopt,text="Load options",activebackground='#00ff00',
                      command=lambda w=keyopt: self.keymreact(1,before,win=w),bg='#afffaf')   
      execute.grid(row=6,column=1,sticky=E)       
      #discard changes
      goback=Button(keyopt,text="Ignore changes",activebackground='#ff0000',
                     command=lambda w=keyopt: self.keymreact(0,before,win=w),bg='#ffafaf')
      goback.grid(row=6,column=0,sticky=W)   
         
      #ensure only one instance of this window
      keyopt.initial_focus = execute
      keyopt.grab_set()
      if not keyopt.initial_focus:
        keyopt.initial_focus = keyopt
      keyopt.initial_focus.focus_set()
      return
   
   #specific
   #run key changes
   def  keymreact(self,cont,b,win=None):
      #always close window
      win.destroy()
      
      #keep changes
      if cont:    
         #keyshape or # of key entries
         if self.keyshape != b[0] or self.numkey.get() != b[1]:
            self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )  
            
         #key size or keyspacing
         if self.keysz.get() != b[2] or self.keysp.get() != b[3]:
            self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
            
         #key values
         if self.keyvnum.get() != b[4]:
            self.runMK(0,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
            
         #tie key to sliders
         if self.thrshctrlkey.get() != b[5]:
            #redraw key and normals
            self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
            allnorm =  self.c.find_withtag('normal')
            self.runCS(allnorm)
            
      #get rid of changes, return to before
      else:
         self.keyshape=b[0]    ; self.numkey.set(b[1])  ;   self.keysz.set(b[2])
         self.keysp.set(b[3])  ; self.keyvnum.set(b[4]) ;   self.thrshctrlkey.set(b[5])
      return     
         
   #specific
   #menu for rigid body & tickmark options
   def mkrbm(self):
      #initial list
      before=[self.wantregion.get(),self.lorp.get(),self.halo.get(),self.rblsz.get(), 
         self.wanttick.get(), self.ticksrt.get(), self.ticklen.get(), self.tickthk.get(),self.tickshade]
      
      rbopt=Toplevel(root)
      rbopt.transient(root)
      rbopt.title('Rigid Body/Tickmark Options')
      rbopt.geometry('+%d+%d' % (root.winfo_rootx() + 200,root.winfo_rooty() + 70))
      
      #------ rigid body options ------
      #display?
      Checkbutton(rbopt,text='Display rigid body regions?',variable=self.wantregion,
          onvalue=True,offvalue=False).grid(row=0,sticky=W)
          
      #polygon or line
      Radiobutton(rbopt,text='Display regions as outlines',variable=self.lorp,value=int(0),
                anchor=W).grid(row=1,sticky=W)
      Radiobutton(rbopt,text='Display regions as open lines',variable=self.lorp,value=int(2),
                anchor=W).grid(row=2,sticky=W)
      Radiobutton(rbopt,text='Display regions as polygons',variable=self.lorp,value=int(1),
                anchor=W).grid(row=3,sticky=W)
           
      #halo
      Scale(rbopt,from_=1,to=30,resolution=int(1),orient='horizontal',variable=self.halo,
             ).grid(row=4,column=1,sticky=W+E)
      Label(rbopt,text='Distance from bases to edge of region',bd=3,bg='white',
             anchor=W).grid(row=4,column=0,sticky=NW+E)
      
      #edge thickness
      Scale(rbopt,from_=1,to=30,resolution=int(1),orient='horizontal',variable=self.rblsz,
             ).grid(row=5,column=1,sticky=W+E)
      Label(rbopt,text='Thickness of line for outlines',bd=3,bg='white',
             anchor=W).grid(row=5,column=0,sticky=NW+E)
             
      #-----tick mark options
      #use tick values?
      Checkbutton(rbopt,text='Display tick markers ?',variable=self.wanttick,
          onvalue=True,offvalue=False).grid(row=6,sticky=W) 
          
      #where ticks start
      Scale(rbopt,from_=1,to=40,resolution=int(1),orient='horizontal',variable=self.ticksrt,
             ).grid(row=7,column=1,sticky=W+E)
      Label(rbopt,text='Distance from bases to tick mark',bd=3,bg='white',
             anchor=W).grid(row=7,column=0,sticky=NW+E) 
             
      #length of tick marks
      Scale(rbopt,from_=1,to=30,resolution=int(1),orient='horizontal',variable=self.ticklen,
             ).grid(row=8,column=1,sticky=W+E)
      Label(rbopt,text='Length of tick mark',bd=3,bg='white',
             anchor=W).grid(row=8,column=0,sticky=NW+E)
             
      #tick mark thickness
      Scale(rbopt,from_=1,to=10,resolution=int(1),orient='horizontal',variable=self.tickthk,
             ).grid(row=9,column=1,sticky=W+E)
      Label(rbopt,text='Tick mark thickness',bd=3,bg='white',
             anchor=W).grid(row=9,column=0,sticky=NW+E)  
             
      #tick mark shade
      garbage = int(0)
      Label(rbopt, text='Shade of all tick marks',bd=3,bg='white',anchor=W
           ).grid(row=10,column=0,sticky=NW+E)
      worthlessname=Scale(rbopt,orient='horizontal',from_=0,to=255,bg='white',variable=garbage,
           command=lambda x=garbage, y=2:self.chcstr(x,y))
      worthlessname.grid(row=10,column=1,sticky=W+E)
      worthlessname.set(int(self.tickshade[1:3],16))
             
      #keep changes
      execute=Button(rbopt,bg='#afffaf',activebackground='#00ff00',text="Load changes",
               command=lambda w=rbopt: self.rbmreact(1,before,win=w))
      #ignore changes
      goback=Button(rbopt,bg='#ffafaf',activebackground='#ff0000',text="Ignore changes",
               command=lambda w=rbopt: self.rbmreact(0,before,win=w))   
      
      execute.grid(row=11,column=1,sticky=E)
      goback.grid(row=11,column=0,sticky=W)
      
      #ensure only one instance of this window
      rbopt.initial_focus = execute
      rbopt.grab_set()
      if not rbopt.initial_focus:
         rbopt.initial_focus = rbopt
      rbopt.initial_focus.focus_set()            
      return
      
   #specialized
   #reacts to change in rigid body menu
   def rbmreact(self,cont,b,win=None):
      #always close window
      win.destroy()
         
      #keep changes
      if cont:
         #if dont want rb regions
         if self.wantregion.get() != b[0]: 
            if self.wantregion.get() == False: self.c.delete('allrb')
            else:
               #make sure have graphpoly file
               if len( self.polyfn.get() ) != 0:
                  #check for precursor files
                  if self.rbfn.get() != 0 and len( self.lrfn.get() ) != 0:
                     self.leftline,self.rightline,self.rbgroups,self.rbpoints=self.rbcalc(self.rbfn.get(),self.Creslist,self.halo.get(),self.lrfn.get(),self.polyfn.get(),self.xyfn.get() )
                  #fails: set back to fail
                  else: self.wantregion.set(False)
                  self.graphpoly(self.rbpoints,self.lorp.get(),self.rblsz.get(),self.xpolm.get(),self.ypolm.get())
               #fails set to false
               else: self.wantregion.set(False)
               
         #if halo changes
         if self.halo.get() !=  b[2] and self.wantregion.get() == True and len( self.polyfn.get() ) != 0:
            self.leftline,self.rightline,self.rbgroups,self.rbpoints=self.rbcalc(self.rbfn.get(),self.Creslist,self.halo.get(),self.lrfn.get(),self.polyfn.get(),self.xyfn.get() )
            
         #if want display, and anythin else changes, re graph
         if self.wantregion.get() == True:
            if self.lorp.get() != b[1] or self.halo.get() != b[2] or self.rblsz.get() != b[3]:
               self.graphpoly(self.rbpoints,self.lorp.get(),self.rblsz.get(),self.xpolm.get(),self.ypolm.get())
          
         #if the state of wanting ticks changed to off
         if self.wanttick.get() != b[4]: 
            self.calltickandnum()   
            
         #if anthing about the ticks changed, re-graph them
         if self.ticksrt.get() != b[5] or self.ticklen.get() != b[6] or self.tickthk.get() != b[7] or self.tickshade != b[8]:
            self.calltickandnum()
            
      else:
         self.wantregion.set(b[0])  ;  self.lorp.set(b[1])
         self.halo.set(b[2])        ;  self.rblsz.set(b[3])
         self.wanttick.set(b[4])    ;  self.ticksrt.set(b[5])
         self.ticklen.set(b[6])     ;  self.tickthk.set(b[7])
         self.tickshade = b[8]
      return
         
   #specialized
   #creates advanced menu
   def mkavm(self):
      #inital states
      before=[self.thrshctrl.get(),self.bondshade,self.xletm.get(),self.yletm.get(),
      self.xsymm.get(),self.ysymm.get(),self.xbndm.get(),self.ybndm.get(),
      self.xpolm.get(),self.ypolm.get(),self.arrowclr.get(),self.arrowthk.get(),
      self.arrowsrt.get(),self.arrowlen.get()]
      
      advopt=Toplevel(root)
      advopt.transient(root)
      advopt.title('Advanced Options')
      advopt.geometry('+%d+%d' % (root.winfo_rootx() + 200,root.winfo_rooty()))
      
      #selected arrows exxistance/color
      Scale(advopt,orient='horizontal',from_=-0.05,to=1.05,resolution=0.01,bg='white',
            variable=self.arrowclr).grid(row=0,column=1,sticky=W+E)
      Label(advopt,text='Select arrows existance/color',bd=3,bg='white',
           anchor=W).grid(row=0,column=0,sticky=NW+E)
           
      #arrow body thickness
      Scale(advopt,orient='horizontal',from_=0.1,to=30,resolution=0.1,bg='white',
            variable=self.arrowthk).grid(row=1,column=1,sticky=W+E)
      Label(advopt,text='Select arrows thickness',bd=3,bg='white',
           anchor=W).grid(row=1,column=0,sticky=NW+E)  
           
      #arrow head start
      Scale(advopt,orient='horizontal',from_=1,to=30,resolution=1,bg='white',
            variable=self.arrowsrt).grid(row=2,column=1,sticky=W+E) 
      Label(advopt,text='Select arrows distance',bd=3,bg='white',
           anchor=W).grid(row=2,column=0,sticky=NW+E) 
        
      #arrow length
      Scale(advopt,orient='horizontal',from_=1,to=60,resolution=1,bg='white',
            variable=self.arrowlen).grid(row=3,column=1,sticky=W+E)
      Label(advopt,text='Select arrows length',bd=3,bg='white',
           anchor=W).grid(row=3,column=0,sticky=NW+E)
      
      #threshold resolution
      Scale(advopt,orient='horizontal',from_=-5,to=5,resolution=1,bg='white',
           variable=self.thrshctrl).grid(row=4,column=1,sticky=W+E)
      Label(advopt,text='Resolution of threshold sliders',bd=3,bg='white',
           anchor=W).grid(row=4,column=0,sticky=NW+E)
             
      #bond shade
      Label(advopt,text='Shade of all bonds',bd=3,bg='white',anchor=W
           ).grid(row=5,column=0,sticky=NW+E)
      garbage=int(0)
      this=Scale(advopt,orient='horizontal',from_=0,to=255,bg='white',variable=garbage,
           command=lambda x=garbage,y=1:self.chcstr(x,y))
      this.grid(row=5,column=1,sticky=W+E)
      this.set(int(self.bondshade[1:3],16))
                 
      #residue letter postitions
      Label(advopt,text='X & Y Adjustments for letter residues',bd=3,bg='white',anchor=W
           ).grid(row=6,column=0,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.xletm
           ).grid(row=6,column=1,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.yletm
           ).grid(row=7,column=1,sticky=NW+E)
           
      #residue symbol postions
      Label(advopt,text='X & Y Adjustments for symbol residues',bd=3,bg='white',anchor=W
           ).grid(row=8,column=0,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.xsymm
           ).grid(row=8,column=1,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.ysymm
           ).grid(row=9,column=1,sticky=NW+E)
           
      #bond adjustments
      Label(advopt,text='X & Y Adjustments for bonds',bd=3,bg='white',anchor=W
           ).grid(row=10,column=0,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.xbndm
           ).grid(row=10,column=1,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.ybndm
           ).grid(row=11,column=1,sticky=NW+E)    
           
      #polygon adjustments
      Label(advopt,text='X & Y Adjustments for rigid body',bd=3,bg='white',anchor=W
           ).grid(row=12,column=0,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.xpolm
           ).grid(row=12,column=1,sticky=NW+E)
      Scale(advopt,orient='horizontal',from_=-10,to=10,bg='white',variable=self.ypolm
           ).grid(row=13,column=1,sticky=NW+E) 
           
      #keep changes
      execute=Button(advopt,bg='#afffaf',activebackground='#00ff00',text="Load changes",
               command=lambda w=advopt: self.advmreact(1,before,win=w))
      #ignore changes
      goback=Button(advopt,bg='#ffafaf',activebackground='#ff0000',text="Ignore changes",
               command=lambda w=advopt: self.advmreact(0,before,win=w))  
      execute.grid(row=15,column=1,sticky=E)
      goback.grid(row=15,column=0,sticky=W) 
        
      #ensure only one instance of this window
      advopt.initial_focus = execute
      advopt.grab_set()
      if not advopt.initial_focus:
         advopt.initial_focus = advopt
      advopt.initial_focus.focus_set()            
      return    
         
   #specilized
   #carries out advanced changes
   def advmreact(self,cont,b,win=None):
      #always close previous window
      win.destroy()
       
      #do changes
      if cont:
      
         #changes for selected arrow
         if self.arrowclr.get() != b[10] or self.arrowthk.get() != b[11] or self.arrowsrt.get() != b[12] or self.arrowlen.get() != b[13]:
            #get current residue x and y
            newx = self.Creslist[self.RCreslut[self.curselres]][2]
            newy = self.Creslist[self.RCreslut[self.curselres]][3]
            self.drawXarrows(newx,newy,self.arrowsrt.get(),self.arrowlen.get(),self.arrowthk.get(),self.arrowclr.get() )
         
         #for resolution
         if self.thrshctrl.get() != b[0]:
            #recalc new resolution
            sigfig=float(1)/( pow(10,self.thrshctrl.get()) )
            #change resolution
            self.minscale.configure(resolution=sigfig)
            self.maxscale.configure(resolution=sigfig)
            #use new resolution to make min/max values
            self.minscale.configure(from_=self.Cmin,to=self.Cmax)
            self.maxscale.configure(from_=self.Cmin,to=self.Cmax)
            
         #for bond shade change
         #wont work for no bonds
         if self.bondshade != b[1]:
            #get all bars bonds handles
            allbonds = self.c.find_withtag('allbarbond')
            for each in allbonds:
               self.c.itemconfigure(each,fill=self.bondshade)
            #get all circle bonds
            allbonds = self.c.find_withtag('allcbond')
            for each in allbonds:
               self.c.itemconfigure(each,outline=self.bondshade)
               #if center colored, recolor it
               value=self.c.itemcget(each,'fill')
               if value != '':
                   self.c.itemconfigure(each,fill=self.bondshade)
                   
         #for letter shove
         if self.xletm.get() != b[2] or self.yletm.get() != b[3]:
            #make sure there are residues to graph
            if len(self.Creslist) > int(1):
               alllets = self.c.find_withtag('reslet')
               self.runCS(alllets)
               self.calltickandnum()
            
         #for symbol shove
         if self.xsymm.get() != b[4] or self.ysymm.get() != b[5]:
            #make sure there are residues to graph
            if len(self.Creslist) > int(1):
               allsym =  self.c.find_withtag('ressym')
               self.runCS(allsym)
               self.calltickandnum()
               
         #for bond shove
         if self.xbndm.get() != b[6] or self.ybndm.get() != b[7]:
            #make sure theres something to show
            if len(self.xybarlist) > int(0) and len(self.xycblist) > int(0):
              self.dspbonds(self.xybarlist,self.xycblist,self.xbndm.get(),self.ybndm.get(),color=self.bondshade)
            
         #for polygon moves
         if self.xpolm.get() != b[8] or self.ypolm.get() != b[9]:
            if len(self.rbpoints) > int(3):
               self.graphpoly(self.rbpoints,self.lorp.get(),self.rblsz.get(),self.xpolm.get(),self.ypolm.get())
                    
      #ignore changes
      else:
         self.thrshctrl.set(b[0]) ; self.bondshade = b[1] ; self.xletm.set(b[2])
         self.yletm.set(b[3]) ; self.xsymm.set(b[4]) ; self.ysymm.set(b[5])
         self.xbndm.set(b[6]) ; self.ybndm.set(b[7]) ; self.xpolm.set(b[8])
         self.ypolm.set(b[9]) ; self.arrowclr.set(b[10]) ; self.arrowthk.set(b[11])
         self.arrowsrt.set(b[12]) ; self.arrowlen.set(b[13])

         
################ other methods ########################

   #specilized
   #for the two user modifiable files, reloads after changes
   def refire(self, switch,win=None):
      #destroy window
      win.destroy()
      
      #for rb reload
      if switch == 1 and self.polyfn.get() != '':
         self.rbpoints = self.readpolyfile(self.polyfn.get() )
          
      #for lr reload
      elif switch == 0:
         if self.rbfn.get() != '' and self.polyfn.get() != '' and self.xyfn.get() != '':
            self.leftline,self.rightline,self.rbgroups,self.rbpoints=self.rbcalc(self.rbfn.get(),self.Creslist,self.halo.get(),self.lrfn.get(),self.polyfn.get(),self.xyfn.get(),detLR=int(0) )
      #regraph
      if self.rbpoints != [] and self.wantregion.get() == True:  
           self.graphpoly(self.rbpoints,self.lorp.get(),self.rblsz.get(),self.xpolm.get(),self.ypolm.get())   
      return

   #specific
   def calltickandnum(self):
      #only call maketick if center list exists
      if len(self.Creslist) == 0:
         return
      else:
         if self.wanttick.get() == True:
            #if we want ticks shown
            self.maketick(self.Creslist,self.ticksrt.get(),self.ticklen.get(),thick=self.tickthk.get(),color = self.tickshade )
         else:
            self.maketick(None,0,0)
   
   #specific
   #runs though the list, finds certain tags for each item in thelist, and applies
   #appropriate shift to the first point of the tick mark
   def refind(self,thelist):
      for lyne in thelist:
         tags=self.c.gettags("res#:" +str(lyne[2]))
         
         #find what ttype of residue it is, and apply correct shift
         if 'reslet' in tags:
            lyne[0] = lyne[0] + self.xletm.get()
            lyne[1] = lyne[1] + self.yletm.get()
            lyne[4] = lyne[4] + self.xletm.get()
            lyne[5] = lyne[5] + self.yletm.get()
            
         elif 'ressym' in tags:           
            lyne[0] = lyne[0] + self.xsymm.get()
            lyne[1] = lyne[1] + self.ysymm.get()
            lyne[4] = lyne[4] + self.xsymm.get()
            lyne[5] = lyne[5] + self.ysymm.get()
         else:
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.refind') 

      return thelist  
         
   #general
   def maketick(self,points,startdist,tickdist,thick=3,color='black' ):
      #destroy all ticks
      self.c.delete('allticks')
      
      #check if want ticks
      if type(points) == list:
         #use makeLR with base list and starting distance
         #need to use full list of bases, for "true" shape
         #lut should be {resid} = [x,y]
         leftlut,rightlut=self.makeLR(points,dist=startdist)
         
         #create lists of xy pairs of residues in order
         #must be in order, not using iterkeys
         leftlist=[]
         rightlist=[] 

         #add left of point and point data into new entry
         leftlist=[ leftlut[line[0]] + line for line in points ]
         rightlist=[ rightlut[line[0]] + line for line in points ]
         
         #determine "outside"
         whichone = self.surveyor(leftlist,rightlist)
         if whichone == int(1):
            correctlist=leftlist
         elif whichone == int(2):
            correctlist=rightlist
         else:
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.maketick') 
            return
         
         #find every tenth residue
         shortlist=[ line for line in correctlist if line[2]%int(10) == 0 ]
         #grab xy values for each residue, and shift, and add to xy values
         shortlist = self.refind(shortlist)
         
         #for items in new short list, create ticks and name them
         tdsq=tickdist**2
         for line in shortlist:
            dely=line[1]-line[5]
            delx=line[0]-line[4]
           
            #if normal
            if delx*dely != int(0):        
               slope = dely / delx
            
               #first xy is actually tick starting point
               ans = sqrt( tdsq/(1+slope**2) )
               
               if delx < int(0):
                  farx = line[0] - abs(ans)
               else:
                  farx = line[0] + abs(ans)
               
               if dely < int(0):
                  fary = -1*abs(slope) *ans + line[1]
               else:
                  fary = line[1] + abs(slope)*ans
                           
            #if rise = 0 = horizontal ticks
            elif dely==int(0):
               fary=line[1]
               if delx < int(0):
                  farx=line[0]-tickdist
               else:
                  farx=line[0]+tickdist
                  
            #if run = 0 = vertical residues
            elif delx==int(0):
               farx=line[0]
               if dely < int(0):
                  fary = line[1] - tickdist
               else:
                  fary = line[1] + tickdist
            else:
               showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.maketick') 


            #create tick
            swath=line[0],line[1],farx,fary
            self.c.create_line(swath,width=thick, capstyle="round",fill = color,
                tags=('tick'+str(line[2]),'allticks'))

   #specilized
   #toggles between distance display or B column
   #runs stuff because of change
   def bdisttog(self,force=0):
      #make sure change
      if self.BcolLdGd.get() != self.lastBcolLdGd or force==1:
         #for switch to Bcol
         if self.BcolLdGd.get() == 0:
            self.drawXarrows(1,1,1,1,1,float(-1) )
            self.curselres = 0
            self.Clut = self.colorlut
            self.changethreshold(self.BcolLdGd.get())
            self.runSC(0)
            self.lastBcolLdGd = self.BcolLdGd.get()
            allres = self.c.find_withtag('allres')
            self.runCS(allres )
            self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
            
         #for switch to distance
         elif self.BcolLdGd.get() == 1: 
            if self.pdbfn.get() != '':
               #find first pdb present residue
               found=False
               while found == False:
                  for blank,res in enumerate(self.RCreslut):
                     if self.xyzdistlut.has_key(res):
                        found=True
                        break
               #create new current lut         
               self.Clut,self.Cmax,distmin=self.dist3D(int(res),self.xyzdistlut)
               #reset sliders and RClut       
               self.lastBcolLdGd = self.BcolLdGd.get()
               #display default new residue
               newx = self.Creslist[self.RCreslut[res]][2]
               newy = self.Creslist[self.RCreslut[res]][3]
               outline=str(self.Creslist[self.RCreslut[res]][0]) + '   '+ str(self.Creslist[self.RCreslut[res]][1])
               self.selectedlabel.configure(text="Selected residue: %s " % outline)
               self.drawXarrows(newx,newy,self.arrowsrt.get(),self.arrowlen.get(),self.arrowthk.get(),self.arrowclr.get() )
               self.curselres = int( res) 

               #change all distances
               allres = self.c.find_withtag('allres')
               self.changethreshold(self.BcolLdGd.get())
               self.runCS(allres )
               self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
          
            #if want distance, but no pdb file
            else:
               self.BcolLdGd.set(self.lastBcolLdGd)
             
         else:
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.bdisttog') 
      return

   #generalized
   #uses selected residue/key, that does exist, and creates distance list from that one
   #inputs are existing key num, and [res]=list(xyz)
   def dist3D(self,origin,table):
      outlut={}
      mlist=[]
      #set origine values
      outlut[origin]=int(0)
      a,b,c=table[origin]
   
      #run through each lut entry, calc dist
      for res in table:
         D=sqrt( pow((a-table[res][0]), 2) + pow((b-table[res][1]), 2) + pow((c-table[res][2]), 2)  )
         outlut[res]=D
         mlist.append(D)
      maxlut=max(mlist)
      minlut=min(mlist)
      return outlut,maxlut,minlut
      
   #specilized
   #converts scale change for bond color in mkavm to color string for comparison in advmreact
   def chcstr(self,it,which):
      #convert int -> hex string
      newsmall=str(hex(int(it))[2:])
      #multiply string up to right color length
      if len(newsmall) == 1:
          newlarge='#'+newsmall*6
      elif len(newsmall) == 2:
          newlarge='#'+newsmall*3
      else:
         showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.chcstr') 
         
      #change variable
      if which == 1:
         self.bondshade = newlarge
      elif which ==2:
         self.tickshade = newlarge
   
   #specialized -> uses strRgb
   #takes a 'center' point, and start/end distance, and line width, and color
   #uses color to choose to graph or not, uses included color mehtod to get color
   #then graphs an 'X' of arrows pointing at center point
   def drawXarrows(self,cx,cy,startd,alen,thk,colorvalue=float(-0.1) ):
      #destroy all pre-existaing arrows, no matter what
      self.c.delete('allarrow')
      #only continue if color >= 0.0
      if not colorvalue < float(0.0):
         #get color, if > 1, black
         if colorvalue > float(1.0):
            paint='black'
         else:
            paint = self.strRgb(colorvalue,int(0),int(1) )
            
         #one time values
         var=float(0.7071067) * startd #project angled distance
         var2=( float(0.7071067) * alen ) + var
         counter=int(1)
         
         #loop to draw four arrows
         while counter < int(5):
            #compute X
            if counter < float(2.5): 
               startx = cx - var
               endx = cx - var2
            else: 
               startx = cx + var
               endx = cx + var2
               
            #compute Y
            switch = counter % int(2)
            if switch:
               starty = cy - var
               endy = cy - var2
            else:
               starty = cy + var
               endy = cy + var2
              
            #invoke line draw
            self.c.create_line(startx,starty,endx,endy,width=thk,fill=paint,
                     arrow='first',tags=('arrow'+str(counter),'allarrow')  )
            self.c.lower('allarrow')
            counter = counter + 1
         
         
   #general
   #display of positional residues 
   #uses later "change" methods to display as it should be initially
   def dsppositions(self,reslist,letter,txsz=5,which=0,ssz=5):
      #delete previous residues, if any
      self.c.delete('allres')
      
      #display residues as letters
      if letter == True:
         for item in reslist:
	    tlist=('res#:'+str(item[0]),item[1],'reslet','allres','notexist')
            self.c.create_text(item[2]+self.xletm.get(),item[3]+self.yletm.get(),text=item[1],
                   font="Helvetica "+str(txsz),tags=tlist)  #+" bold"
	    
      #display residues as symbols
      else:
	 for item in reslist:
	    tlist=('res#:'+str(item[0]),item[1],'ressym','allres','notexist')
	    self.shape(which,item[2]+self.xsymm.get(),item[3]+self.ysymm.get(),ssz,'black',tlist)
      return

   #general
   #to display groupings by rigid body, colored background of polygons or loops
   def graphpoly(self,data,lorp=int(0),linsize='2.0',xshove=0,yshove=0):
      #to graph the border line/shape
      #lorp=0=line,=1=polygon
      #data[entry]=[name,color,list[x],list[y]]
      lnsz = float(linsize)
      
      #remove all previous rigid bodies
      self.c.delete('allrb')
      
      #loop through every polygon
      for thing in data:
         names = ('rbid:'+str(thing[0]),'allrb')
         colorchoice = thing[1]
         
         posx=map(lambda x:x+int(xshove),thing[2])
         posy=map(lambda x:x+int(yshove),thing[3])
         
         #intersperse x and y together
         xylist=[]
         counter=0
         while counter < len(posx):
            xylist=xylist+[ posx[counter],posy[counter] ]
            counter = counter + 1
            
         #decide if polygon or edge
         if lorp == 0: centercolor=''
         else:centercolor = colorchoice
                  
         #create polygon/edge
         if lorp < int(2):
            self.c.create_polygon(xylist,outline=colorchoice,fill=centercolor,
               width=lnsz,tags=names)
         #for open line
         elif lorp == int(2):
            self.c.create_line(xylist,fill=colorchoice,width=lnsz,tags=names)
         else:
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.graphpoly') 
               
      #move all polygons to below bases/bonds
      self.c.lower('allrb')  
      
   #general -- borrowed from Tapu
   #to never write over a file
   def backup(self,filename):
      if os.path.exists(filename):
        found_vacancy = 0
        tiebreaker = 0
        shortdir = os.path.basename(os.path.dirname(filename))
        while not found_vacancy:
            test_filename = filename + '_' + str(tiebreaker)
            if os.path.exists(test_filename):
                tiebreaker += 1
            else:
                found_vacancy = 1
                short_old = os.path.join(shortdir, os.path.basename(filename))
                short_new = os.path.join(shortdir, os.path.basename(test_filename))
                showwarning(title="File Moved", message = str(short_old)+" moved to "+str(short_new))
                os.rename(filename, test_filename)  
   #specific
   def prank(self):
         showwarning(title="This program's truename is:", message='Coloring Residues and Annotating '
             +'Secondary Structures, CRASS. This however, was deemed too uncouth for a program name, '
             +'and for no good reason given its more popular name. '
             +"Don't you agree CRASS would've been a better name?")
             
############## reaction to mouse events ####################
      
   #what to do on button push if on residue that does something
   def OnCanvasMouseDown(self,event):
      #if mouse down on ANYTHING
      curlist = self.c.find_withtag(CURRENT)
      if curlist:
         tags = self.c.gettags(CURRENT)

         #if mouse over a residue
         if 'allres' in tags:
            outline = tags[0][5:] +"  "+tags[1]
            
            #check if residue has 3d info and want dist? if so run distClut calc
            if self.BcolLdGd.get() == int(1) and self.xyzdistlut.has_key(int(tags[0][5:]) ):
               self.selectedlabel.configure(text="Selected residue: %s " % outline)
               self.drawXarrows(event.x,event.y,self.arrowsrt.get(),self.arrowlen.get(),self.arrowthk.get(),self.arrowclr.get() )
               self.Clut,self.Cmax,distmin=self.dist3D(int(tags[0][5:]),self.xyzdistlut)
               self.curselres = int(tags[0][5:])
               self.changethreshold(self.BcolLdGd.get(),self.Cmax)
               self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
               self.runSC(0)
               allnorm = self.c.find_withtag('normal')
               self.runCS(allnorm)

         #if mouse over key 
         if 'allkey' in tags:
            self.xmoveold = event.x
            self.ymoveold = event.y
      else:
         #selected whitespace
         self.selectedlabel.configure(text="Selected residue: ")
         self.curselres = 0
         self.drawXarrows(1,1,1,1,1,float(-1) )
            
   #what to do if mouse moves find mouseover residue, display in curent label
   def OnCanvasMouseMove(self,event):
      #if mouse over on ANYTHING
      curlist = self.c.find_withtag(CURRENT)
      if curlist:
         tags = self.c.gettags(CURRENT)
         
         #if mouse over a residue
         if 'allres' in tags:
            outline = tags[0][5:] +"  "+tags[1]
            self.currentlabel.configure(text="Current residue: %s " % outline)
            #if residue has entry in CLut or not
            if self.Clut.has_key(int(tags[0][5:])):
               number = float( self.Clut[int(tags[0][5:])] )
               #find decimal, for/backwards compatibly
               value = round(number,self.thrshctrl.get())
                     
            else: value='none'
            self.currentvalue.configure(text="Current value: %s " % value)
            
      else:
         self.currentlabel.configure(text="Current residue: ")
         self.currentvalue.configure(text="Current value: ")
         #self.prank() #What the heck is this thing?

      return
      
   #for button move while pressed
   def OnCanvasMouseDrag(self,event):
      #if mouse down on ANYTHING
      curlist = self.c.find_withtag(CURRENT)
      if curlist:
         tags = self.c.gettags(CURRENT)
         
         #for mouse over key
         if 'allkey' in tags:
            self.c.move('allkey',event.x - self.xmoveold, event.y - self.ymoveold)
            self.xmoveold = event.x
            self.ymoveold = event.y

############ idividual reactions to slider/buttons ###############
      
   #specific
   #from letter/symbol toggle calls dsppositions if there is a change
   def letnumtog(self):
      if self.resletnum.get()!=self.lastresletnum:
         #change value
         self.lastresletnum=self.resletnum.get()
         #get list of all residues
         allres = self.c.find_withtag('allres')
         #change all residues            
         self.runSC(int(0) )      
           
   #specific
   #changes ranges and "meaning" of threshold bars
   def changethreshold(self,which,high=0):
      #for pdb B-column
      if which == 0:
         self.Cmin=self.pdbmin
         self.Cmax=self.pdbmax
         #make sure color is current look up table
	 self.Clut=self.colorlut
      #for distance
      if which == 1:
         self.Cmin=int(0)
         #self.Cmax=high done by dist3D call
         #Clut fix should be reassigned already
          
      #set min thresh low and high and thresh itself
      self.minthresh.set(self.Cmin)
      self.minscale.configure(from_=self.Cmin,to=self.Cmax)
      #set max thresh low&high and thresh itself
      self.maxscale.configure(from_=self.Cmin,to=self.Cmax)
      self.maxthresh.set(self.Cmax)
      #reset previsou values
      self.lastminthresh=self.minthresh.get()  
      self.lastmaxthresh=self.maxthresh.get() 
	       
      #everytime a new Clut is declared, redo reverseClut
      self.RClut={}
      for k,v in self.Clut.iteritems():
         keys=self.RClut.setdefault(v,[])
         keys.append(k)
               
   #specailized
   #keeps threshold bars "where they should"
   def forcethresh(self,newvalue,which):
      value=float(newvalue)
      #for min too high
      if which==0:
         self.maxscale.configure(from_=value)	    
      #for max too low
      if which==1:
         self.minscale.configure(to=value)
   
   #specialized
   #runs state check and others when sliders are __released__
   def activatescale(self,event=None):
      self.runSC(1)
      #if want key tied to slider values
      if self.thrshctrlkey.get() == True:
         self.runMK(1,self.numkey.get(),self.keysp.get(),self.keyshape,self.keysz.get(),self.keyvnum.get() )
         allnorm =  self.c.find_withtag('normal')
         self.runCS(allnorm)
   
   #specialized
   #flips coloring of residues above/below threshold
   def flipthreshcolor(self,who,currentstate):
      #get all residues looking for
      thresholded=self.c.find_withtag(who)
      #specifiy current color
      if currentstate == 0: color=''
      elif currentstate == 1: color='black'
      else: showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.flipthreshcolor') 
      #I expect problem with bases as letters, no outline attribute
      edge=color
      for each in thresholded:
         self.c.itemconfigure(each,fill=color,outline=edge)
         
############# recalc every residue's state ####################
            
   #spcecific
   #calls state calc
   def runSC(self,which):
    #run and redo all residues
    if which == 0:
      self.statecalc(self.Clut,self.Creslist,self.resletnum.get(),self.minthresh.get(),self.maxthresh.get(),self.reslowthst,self.reshgthst)

    #only redo residues within threshold change, if Creslist is non zero
    elif (which == 1) and ( len(self.Creslist) != int(0) ):
       #default values, used as continuation keys too
       big = int(-1)
       small=int(-2)
       
       #if minthresh changed
       if self.minthresh.get() != self.lastminthresh:
          #designate new big/small
          if self.minthresh.get() > self.lastminthresh:
             big=self.minthresh.get()
             small=self.lastminthresh
          else:
             small=self.minthresh.get()
             big=self.lastminthresh
          #reset lastthresh   
          self.lastminthresh = self.minthresh.get()

       #if max thresh changed
       elif self.maxthresh.get() != self.lastmaxthresh:
          #make new big/small
          if self.maxthresh.get() > self.lastmaxthresh:
             big=self.maxthresh.get()
             small=self.lastmaxthresh
          else:
             small=self.maxthresh.get()
             big=self.lastmaxthresh
          #reset last maxthresh to detect next time
          self.lastmaxthresh = self.maxthresh.get()

       #if anything changed, calc affected residues
       #for affected keys, have list of resid, not list values for compatability with 
       #self.statecalc, need to give in list form, i.e. Creslist
       if big != int(-1) and small != int(-2):
          output = []  #mini copy of Creslist of affected residues
       
          #search through reverse lookup, see if any keys between big and small
          for k,vlist in self.RClut.iteritems():
             if (k <= big) and (k >= small):
              
                #in case of mult. res with exact same value, iter through V
                for each in vlist:
                   #not every residue with a pdb entry has a xy entry
                   if self.RCreslut.has_key(each):
                      output.append(self.Creslist[self.RCreslut[each] ])
          self.statecalc(self.Clut,output,self.resletnum.get(),self.minthresh.get(),self.maxthresh.get(),self.reslowthst,self.reshgthst)       
          
   #general
   #for every change(in LUT, or threshold) calculate new states
   #calcs what each residue should be, compares that to current, 
   #   if different, saves number of item and changes tags to match what should
   #   stategraph redraws only "different" items
   def statecalc(self,lut,res,letsym,minth,maxth,lowdo,highdo):
      #run through all residues, according to reslist
      output=[]
      #optimize?
      lhk = lut.has_key
      newtig=[]
      lst = (letsym == True)
      
      for entry in res:
         #start new tig list for each residue
         newtig=['allres']

	 #existance test
	 #residue exists
	 if lhk(entry[0]):
	    #greater than max thresh
	    if lut[entry[0]] > maxth:
	       newtig.append('above')
	       
	       #if display as other
	       if highdo == int(2):
	          #reverse display
	          if lst: newtig.append('ressym')
		  else: newtig.append('reslet')
	       #display as normal
	       else:
	          if lst: newtig.append('reslet')
	          else:newtig.append('ressym')
		  
	    #less than min thresh
	    elif lut[entry[0]] < minth:
	       newtig.append('below')
	       
	       #if display as other
	       if lowdo == int(2):
	          #reverse display
	          if lst: newtig.append('ressym')
		  else: newtig.append('reslet')
	       #display as normal
	       else:
	          if lst: newtig.append('reslet')
	          else:newtig.append('ressym')
	          
	    #between or equal to either thresh
	    else:
	       newtig.append('normal')
	       #letter or symbol
	       if lst: newtig.append('reslet')
	       else:newtig.append('ressym')
	 
	 #does not exist
	 else: 
	    newtig.append('notexist')
	    #letter or symbol
	    if lst: newtig.append('reslet')
	    else: newtig.append('ressym')
	 
	 #now have newtig, get oldtig(current) for this residue
	 #"name" is the integer handle given by canvas to everything drawn
	 name = self.c.find_withtag('res#:'+str(entry[0]))
         
         #catch for pdb file w/ residues xy file does not
         #normally figure change if any 
         if len(name) == int(1):
            oldtig = self.c.gettags(name)
            
            #give resid and res letter to newnewtiglist
            newnewtig=list(oldtig[:2])+newtig
            
            #intersect lists
            intersect = [ n for n in newnewtig if n in oldtig ]
            
            #if intersection is not same length as newnewtig,
            #then something must have changed
            if len(intersect) != len(newnewtig):
            
               #change that item's tags
               self.c.itemconfigure(name,tags=tuple(newnewtig))
               #save item in list to be changed
               output.append(name)
                        
         #if too many residues
         elif len(name) > int(1):
            #print "More than one residue with "+str(entry[0])+" as a name" 
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.statecalc') 
            break    
         #if too few residues
         else:
            continue
        
      #after calculating action of changing LUT or threshold
      #call changestates to enforce change
      if len(output) > 0:
         self.runCS(output)
      return      
   
   #specific
   #gives the same values to changestates
   def runCS(self,throughput):
     #evaluate not exist letter
     if self.resletnex.get() == '':# or self.resletnex.get() == string.whitespace:
        letterout = False
     else:
        letterout = self.resletnex.get()
     self.changestates(throughput,self.ressymsz.get(),self.resletsz.get(),letterout,self.ressymex,self.ressymnex,self.reslowthst,self.reshgthst)     
   
   #general
   #receives list of changed handles from calcstate, then calls 
   #appropriate redraw graph function
   def changestates(self,handles,symsz,letsz,noletter,existsym,nexistsym,lowdo,highdo):
      #optimize
      hs = (highdo == int(0) )
      ls = (lowdo == int(0) )
      tletter = (type(noletter) == str )
      
      #iterate through changed residues
      for each in handles:
         tags=self.c.gettags(each)
         
         #optimize?
         state=int(-70)
         if 'normal' in tags: state=0
         elif 'above' in tags: state=1
         elif 'below' in tags: state=2
         elif 'notexist' in tags: state=3
         
         #if normal get color 
         if state == 0:
            color=self.runcolor(each)
         #if above or below, and not supposed to show
         elif state == 1 and hs:
            color=''
         elif state == 2 and ls:
            color=''
         else : color='black'

         #for letters
         if 'reslet' in tags:
            letter = tags[1]
                        
            #either the not letter or the standard letter
            if (state == 3) and tletter:
               letter = noletter

            #call primary change grapher
            self.runGSL(each,letter,symsz,letsz,color)
            
         #for symbols
         elif 'ressym' in tags:
            #decide what shape to use
            if state < 3:
               which = existsym
            elif state == 3:
               which = nexistsym
            else:
               showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.changestates') 
               
            #call primary change grapher
            self.runGSL(each,which,symsz,letsz,color)
            
         #error catch
         else: showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.changestates. Type 2') 
      return
             
################ symbol generation and change ####################
   
   #specific
   #calls reshape, and creates look up table for usual classes
   def runreshape(self,through):
      lut={'notexist':self.ressymnex,'normal':self.ressymex,
           'above':self.ressymex ,'below':self.ressymex}
      self.reshape(through,lut,self.ressymsz.get())
   
   #general
   #redraws residues based on tags
   #nametup is tuple of strings to search for in graphs
   def reshape(self,nametup,whattodolut,symsize):
      #only applies to symbol residues
      symbols = self.c.find_withtag('ressym')
      
      #iterate through each "class" afffected
      for name in nametup:
         #create exception for all ressym
         if name != 'ressym':
           #get those types
           currenttup = self.c.find_withtag(name)
           #find intersection of symbols and class
           both = [n for n in currenttup if n in symbols ] 
         else:
           both = symbols        
         
         #for each residue that is a symbol and class
         for handle in both:
            oldfill= 'black' #save for graphsymlett

            #for normal things, get color
            if name == 'normal': #save for graphsymlett
               oldfill=self.c.itemcget(handle,'fill') #save for graphsymlett

            which = whattodolut[name] #save for graphsymlett
            #create new
            self.runGSL(handle,which,symsize,0,oldfill)
            
   #specific
   #used to resize all residues shown as symbols
   def symresize(self):
      #variables
      search = 'ressym'
      
      #find all symbols
      allsym= self.c.find_withtag('ressym')
      for each in allsym:
         oldxy = self.c.coords(each)
         color = self.c.itemcget(each,'fill')
         
         #determine what each one is
         kind=self.c.type(each)
         
         if kind == 'polygon':
            #diamond
            if len(oldxy) == int(8):
               newx= oldxy[0] ; newy= oldxy[3]
               which = int(2)
            #triangle
            elif len(oldxy) == int(6):
               newx= oldxy[4] ; newy= (oldxy[1]+oldxy[5]) /2
               #up
               if oldxy[1] > oldxy[5]: which = int(3)
               else: which = int(4)
            else:
               showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.symresize') 
               
         elif kind == 'rectangle':
            newx= (oldxy[0]+oldxy[2]) /2 ; newy= (oldxy[1]+oldxy[3]) /2
            which = int(1)
         elif kind == 'text':
            newx=oldxy[0]  ;  newy = oldxy[1]
            which = self.c.itemcget(each,'text')
         elif kind == 'oval':
            newx= (oldxy[0]+oldxy[2]) /2 ; newy= (oldxy[1]+oldxy[3]) /2
            which = int(0)
         else:
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.symresize Type 2') 
         
         #create new
         self.runGSL(each,which,self.ressymsz.get(),self.resletsz.get(),color)
   
   #specific
   #runs graphsymlett, auto-gives Creslist, and RCreslist
   def runGSL(self,p1,p2,p3,p4,p5):
      #get current handle id
      rid=int(self.c.gettags(p1)[0][5:])
      
      #if base# exists
      if self.RCreslut.has_key(rid): 
         #using index from RCreslut, get orig xy values
         x=round(self.Creslist[self.RCreslut[rid]][2])
         y=round(self.Creslist[self.RCreslut[rid]][3])
         
         #create different "shoves" for letter or symbols
         #symbols
         if type(p2) == int:
            y=y+self.ysymm.get()
            x=x+self.xsymm.get()
         #letters
         elif type(p2) == str:
            y=y+self.yletm.get()
            x=x+self.xletm.get()
            beer=2
         #error
         else: 
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.runGSL') 
         
      self.graphsymlett(p1,p2,p3,p4,p5,x,y)
      return
            
   #general
   #graphs things one by one, primarily switches residues between syms and letts.
   def graphsymlett(self,handle,what,ssz,lsz,color,newx,newy):
      #get current tags, to give to new
      oldtags = self.c.gettags(handle)
      #destroy old item, get everything I need
      self.c.delete(handle)
           
      #for shapes
      if type(what) == int:
         self.shape(what,newx,newy,ssz,color,oldtags)
      
      #for letters   
      elif type(what) == str:
         self.c.create_text(newx,newy,text=what,anchor=CENTER,font="Helvetica "+str(lsz),
                            fill=color,tags=oldtags)  #+" bold"
      #error
      else:
         showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.graphsymlett') 
      return
      
############### getting file names ########################
      
   #specialized--returns "" if user hits 'cancel',for file opening
   def askfilename(self,topsays,switch=int(0)):
     " if you hit 'cancel', askopenfilename returns an empty tuple "
    
     #for bills use/open previous files
     if switch == int(0):
       f = askopenfilename(title=topsays)
     #my new use/open files that dont exist yet
     elif switch == int(1):
       f = asksaveasfilename(defaultextension=".txt",title = topsays)
     #new files with extension of .ps  
     elif switch == int(2):
       f = asksaveasfilename(defaultextension=".eps",title = topsays)
     #error catch
     else:
       showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.askfilename') 
      
     #always running  
     if len(f) == 0:
        return ""
     else:
        f = string.replace(f,"/tmp_mnt","")
        return f 
	
   #specilized --to open other needed files 1 for lr and polygon, 2 for tnt and poly
   def getmorefiles(self,dowhat):
     #to open new leftright/new polygon
     if dowhat == int(1):
       fn1 = self.askfilename("Enter name for new left/right file now:",switch = int(1))
       fn2 = self.askfilename("Enter name for new colored groups file now:",switch = int(1))
       return fn1,fn2
       
     #to open old realspace/newpolygon file
     if dowhat == int(2):
       fn1 = self.askfilename("Select real-space TNT file now:")
       #run defined method
       if len(fn1) == 0:
         return
       if not os.path.exists(fn1):
         #print "cannot find %s" % fn1 
         showwarning(title='coloRNA: File Not Found', message='Can not find file named'+str(fn1)) 
         return
       fn2 = self.askfilename("Enter name for new colored groups file now:",switch = int(1))
       return fn1,fn2
       
     #save canvas as postscript
     if dowhat == int(3):
        fn = self.askfilename("Name for PostScript file:",switch=int(2))
        #if closed out, do nothing
        if len(fn) == 0:
           return
        #if a name and a path
        else:
           self.c.postscript(file=fn)               
       	
   #used to change name variables for both pdb and xy file 1.0 = pdb, 2.0 = xyinfo    
   def getfile(self, whichfile,filename=None):
        
	#make dictonary so title of popup says something
	saydict ={int(1):"pdb",int(2):"xy-data",int(3):"rigid body TNT",
	             int(4):"left/right",int(5):"colored groups"}
                     
	tl = "Select "+saydict[whichfile]+" file now:"
	
	#run defined method
        filename = self.askfilename(tl)
        if len(filename) == 0:
            return
        if not os.path.exists(filename):
            #print "cannot find %s" % filename 
            showwarning(title='coloRNA: File Not Found', message='Can not find file named'+str(filename))
            return    
	    
	#updating/createinfo from pdb    
        if int(whichfile) == int(1.0): self.pdbfn.set(filename)
	   
	#update/create info for xy positions
	elif int(whichfile) == int(2.0): self.xyfn.set(filename)
	
	#update/create from TNT file
	elif int(whichfile) == int(3.0):
	   self.rbfn.set(filename)
	   #get new names
	   blah1,blah2 = self.getmorefiles(int(1)) 
	   self.lrfn.set(blah1)
	   self.polyfn.set(blah2)
	   #run basic methods
	     
	#update/create from L/R file
	elif int(whichfile) == int(4.0):
	   self.lrfn.set(filename)
	   #get other names
	   blah1,blah2 = self.getmorefiles(int(2))
	   self.rbfn.set(blah1)
	   self.polyfn.set(blah2)
	   
	#load pre-made graphics file
	elif int(whichfile) == int(5.0):
	   self.polyfn.set(filename)
           
	else:
	   #print "Did not specify which file to update" 
           showwarning(title='coloRNA: File Problem', message="Did not specify which file to update") 
	   return
	   
############## rigid body processing ########################

   #specialized
   #does the "background" calculations needed for colored regions
   def rbcalc(self,groupsfile,bases,angel,lrfile,pfile,xyfn,detLR =int(1),redogrp=int(1)):
      #?load tnt space file into groups?
      if redogrp == int(1):
        groups = self.readtnt(groupsfile)
       
      #always make left/right lines from bases
      lefttable,righttable = self.makeLR(bases,angel)
     
      #?run determineLR/surveyor?
      if detLR == int(1):
         self.backup(lrfile)
         self.fileheader(lrfile,xyfn,groupsfile)
         validres = self.determineLR(bases,lefttable,righttable,groups,lrfile)
      
      #always create new polygon file
      self.backup(pfile)
      self.fileheader(pfile,xyfn,groupsfile)
      self.makepolygon(lrfile,pfile,groups,lefttable,righttable,aura=angel)
      #read new file into data
      colors = self.readpolyfile(pfile) 
      return lefttable,righttable,groups,colors
      
   #general
   def makepolygon(self,infile,outfile,grps,Llut,Rlut,colorc=0,aura=7):
      #checks for, and creates if need, left and right look up table
      #for valid groups gets left, right points and prints to polygonfile
   
      #error check
      if len(Llut) != len(Rlut):
         showwarning(title='coloRNA Internal Error Check', message='Left & right line uneven in self.makepolygon') 
         return "Error"
      
      #open valid groups file
      startreading = False
      f = open(infile,"r")
      j = open(outfile,"a")

      #read all lines
      for entry in f.readlines():
         words = string.split(entry)
    
         #look for first usable line, first word = 0
         if startreading == False:
            if words[0] == "0":
               startreading = True

         #found first line
         if startreading == True:
            #rename lines for ease, lorr = left or right
            grpindex = int(words[0]) ; lorr = int(words[2])
            
            #check if group "printable" = valid and either left/right
            if ( int(words[1]) == int(1) ) and ( lorr == int(1) or lorr == int(2) ):
            
               #get newcolor and next colorcounter
	       newcolor,colorc = self.colorwheel(colorc)
	       #what residues are we looking for
	       crtgrpres = grps[grpindex]
	
	       #initiate
               linex = ""
	       liney = ""
	       spot=int(0)
               lastres=crtgrpres[0]
               
	       #for left of current group
	       if lorr == int(1):
                  #include other of first
                  linex=linex+str(round(Rlut[crtgrpres[0]][0],3))+"  "
                  liney=liney+str(round(Rlut[crtgrpres[0]][1],3))+"  "
                  
	          #current group place in string
	          while spot < len(crtgrpres) : #for residue in crtgrpres:
	             linex=linex+str(round(Llut[crtgrpres[spot]][0],3))+"  "
	             liney=liney+str(round(Llut[crtgrpres[spot]][1],3))+"  "
                     spot=spot+1
                     #include other of break in res chain
                     if spot < len(crtgrpres): 
                        if lastres != (crtgrpres[spot] - int(1)):
                           linex=linex+str(round(Rlut[crtgrpres[spot-1]][0],3))+"     "
                           liney=liney+str(round(Rlut[crtgrpres[spot-1]][1],3))+"     "
                           linex=linex+str(round(Rlut[crtgrpres[spot]][0],3))+"  "
                           liney=liney+str(round(Rlut[crtgrpres[spot]][1],3))+"  "
                        lastres=crtgrpres[spot]
                     
	          #include other of last
                  linex=linex+str(round(Rlut[crtgrpres[-1]][0],3))+"  "
                  liney=liney+str(round(Rlut[crtgrpres[-1]][1],3))+"  "
                  
	       #for right 
	       elif lorr == int(2):
	          #current group place in string
                  #include other of first
                  linex=linex+str(round(Llut[crtgrpres[0]][0],3))+"  "
                  liney=liney+str(round(Llut[crtgrpres[0]][1],3))+"  "
                  
	          while spot < len(crtgrpres) : #for residue in crtgrpres:
	             linex=linex+str(round(Rlut[crtgrpres[spot]][0],3))+"  "
	             liney=liney+str(round(Rlut[crtgrpres[spot]][1],3))+"  "
                     spot=spot+1
                     #include other of break in res chain
                     if spot < len(crtgrpres): 
                        if lastres != (crtgrpres[spot] - int(1)):
                           linex=linex+str(round(Llut[crtgrpres[spot-1]][0],3))+"     "
                           liney=liney+str(round(Llut[crtgrpres[spot-1]][1],3))+"     "
                           linex=linex+str(round(Llut[crtgrpres[spot]][0],3))+"  "
                           liney=liney+str(round(Llut[crtgrpres[spot]][1],3))+"  "
                        lastres=crtgrpres[spot]
                     
                  #include other of last
                  linex=linex+str(round(Llut[crtgrpres[-1]][0],3))+"  "
                  liney=liney+str(round(Llut[crtgrpres[-1]][1],3))+"  "
                  
	       #problem
               #debug add ability to deal with 3= both
	       else:
                  showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.makepolygon') 
	          return "error"
	    
	       #print three lines for each group
	       line1="%s\n\n"%("real space group "+str(grpindex)+" "+str(newcolor))
	       j.write(line1)
	       j.write("%s\n\n"% linex)
	       j.write("%s\n\n"% liney)
      f.close()
      j.close()	
      return int(1)   

   #general
   def colorwheel(self,crct):
      #loops through colorlist within this method.
      #will write this so that only need to change list
      #returns "current" color and nextcut, to update imput of currentcut
  
      #change this for different color loop/scheme
      cllp=['9FF','F3F','FF9','39F','F9C','9F3','99F','F33','9F9','93F','FC9',
      '3F9','F9F','FF3','9CF','F39','CF9','33F','F99','3F3','C9F','F93','9FC']
      
      #check that input is < len(colorwheel)
      if(crct >=len(cllp)):
         showwarning(title='coloRNA Internal Error Check', message='Input to self.colorwheel too large.') 
         return "error","error"
    
      #normal run
      if(crct < len(cllp)):
         #"doubles" each entry, and tack on #
         colorvalue="#"+cllp[crct][0]*2+cllp[crct][1]*2+cllp[crct][2]*2
    
         crct = crct+1
         if crct == len(cllp):
            crct = 0      
      return colorvalue,crct   
      
   #general
   def determineLR(self,xylist,leftlut,rightlut,groups,outfile):
      #assume that filename is valid
      #creates list of valide residues. Checks if every group's every residue
      #  exists on current plot. If so, determines if "left or right" is "outside"
      #  Prints group#,valid?,left=1,right =2
  
      #check for lengths to be equal
      if len(xylist) != len(leftlut) != len(rightlut):
         showwarning(title='coloRNA Internal Error Check', message="Data input to self.determineLR not equal") 
         return "Error"
    
      #prep to write out to file
      f = open(outfile,"a")  
      #print explanation to file
      s= "%s\n" % ("File contains regions(start at 0) are valid(1),")
      f.write(s)
      s= "%s\n" % ("and either left(1) or right(2)")
      f.write(s)
             
      #loop through all residues  
      validres = range(len(xylist))
      rescount = int(0)
      while rescount < len(xylist):
  
         #create list of res numbers
         validres[rescount] = xylist[rescount][0]
         rescount = rescount + int(1)
  
      #loop through each real space group
      numgroups = len(groups)
      gpcounter = int(0)
      while gpcounter < numgroups:
  
         leftsubmit =[] 
         rightsubmit = []
         #find intersection of currentgroup and all present residues
         intersection =[ n for n in groups[gpcounter] if n in validres ]
    
         #if intersection is same as current group
         if(intersection == groups[gpcounter]):
    
            #for each point in the group,get left and right coords
            for reselement in intersection:
               leftsubmit.append(leftlut[reselement])
	       rightsubmit.append(rightlut[reselement])
            #send to surveyor
            whichone = self.surveyor(leftsubmit,rightsubmit)
      
            #output to file
            s="%d %d %d\n" % (gpcounter,int(1),whichone)
            f.write(s)
      
         else:
            #output to file
            s="%d %d %d\n" % (gpcounter,int(0),int(0))
            f.write(s)
      
         gpcounter = gpcounter + 1
      f.close()    
      return validres 
      
   #general
   def surveyor(self,area1,area2):
     #performs area calculation, left should be first by convention
     #each list item should be a 2d array, [x,y]
     sizefirst =len(area1)
     sizesecond =len(area2)
     sizes =range(1,3)
     mlist =[1,2]
  
     #ensure same length
     if(sizefirst != sizesecond):
        showwarning(title='coloRNA Internal Error Check', message="Two arrays into self.surveyor not equal") 
        return "Surveyor error"
     else:
    
       #loop to save code, fires twice and switches names
       for element in mlist:
          currentlist = eval("area"+str(element))
          currentsize = int(0)
          counter =int(0)
      
          #loop to do calculations
          while counter < (sizefirst - int(1)):
             leta = currentlist[counter][0]
	     letb = currentlist[counter+1][0]
	     letc = currentlist[counter][1]
	     letd = currentlist[counter+1][1]
	
	     currentsize = currentsize +( (leta*letd) - (letb*letc) )
	     counter = counter+1
      
          #end of counting, loop around
          leta = currentlist[counter][0]
          letb = currentlist[0][0]
          letc = currentlist[counter][1]
          letd = currentlist[0][1]
          currentsize = currentsize +( (leta*letd) - (letb*letc) )
          #give values to outside list, for real area halve this value
          sizes[element-1] = abs(currentsize)
      
       #after calculation, determine larger
       #both areas same sign
       if sizes[0]*sizes[1] > int(0):
          if (abs(sizes[0]) > abs(sizes[1])):
             output = int(1)
          elif (abs(sizes[0]) < abs(sizes[1])):
             output = int(2)
       #different signs or zero
       else:
          if (sizes[0] > sizes[1]):
             output = int(1)
          elif (sizes[0] < sizes[1]):
             output = int(2)
          else:
             output = "Error" 
             showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.surveyor')      
     return output 
      
   #general    
   def fileheader(self,output,file1,file2):
      #opens and clears output, prints message with file1 and 2
      #   at top of output,later calls must open("a")
      f1 = open(output,"w")
      f1.write("This file created using files:\n")
      f1.write(file1+"\n")
      f1.write(file2+"\n")
      f1.close()
      return "Zero"
     
   #general
   def makeLR(self,baselist,dist=5):
    #assumes that first and last bases are continuous with their neighbors
    #left and right are arbitrary, but for a circle drawn clockwise,
    #  "left" would be along the outside of the circle
    length = len(baselist)
    leftlist =  range(length)
    rightlist = range(length)
    leftlut = {}
    rightlut = {}
    shouldbe = 2
    counter = int(0)
    garbage =range(2)
 
    #main loop
    while counter < length - 1:
 
      ytwo = float(baselist[counter+1][3])
      yone = float(baselist[counter][3])
      xtwo = float(baselist[counter+1][2])
      xone = float(baselist[counter][2])
 
      #for primary slope
      toppslope = ytwo - yone
      bottompslope = xtwo - xone
 
      #check for horiz
      if toppslope == 0:
        leftx = rightx = xone
        #directionality
        if bottompslope > 0 :
          lefty = yone + dist ; righty = yone - dist
        else:
          lefty = yone - dist ; righty = yone + dist
 
      #check for vert
      elif bottompslope == 0:
       lefty = righty = yone
       #directionality
       if toppslope > 0 :
         leftx = xone - dist ; rightx = xone + dist
       else:
         leftx = xone + dist ; rightx = xone - dist
 
      #all else, usual sslope = secondary slope = neg recip of primary
      else:
        sslope = (-1)*(bottompslope/toppslope)
 
        delx = sqrt( (dist*dist) /( (sslope*sslope)+1) )
        dely = sqrt( (dist*dist) /(1+ (1/(sslope*sslope)) ) )
 
        #direction
        if toppslope > 0:
          leftx = xone - delx ; rightx = xone + delx
        else:
          leftx = xone + delx ; rightx = xone - delx
        #more direction
        if bottompslope > 0:
          lefty = yone + dely ; righty = yone - dely
        else:
          lefty = yone - dely ; righty = yone + dely
 
      #before compare, first values for c+1
      leftlist[counter+1] = [leftx + bottompslope, lefty + toppslope]
      rightlist[counter+1] =[rightx + bottompslope,righty + toppslope]
 
      #check for continuous bases
      if baselist[counter + 1][0] == shouldbe:
 
        if counter > 0: 
          #get previous values
          lastleft = leftlist[counter] ; lastright = rightlist[counter]
 
          #compare
          if lastright[0] != rightx:
            rightx =(lastright[0] + rightx)/2
          if lastright[1] != righty:
            righty =(lastright[1] +righty)/2
          if lastleft[0] != leftx:
            leftx =(lastleft[0] +leftx)/2
          if lastleft[1] != lefty:
            lefty =(lastleft[1] + lefty)/2
 
          #give values to current
          leftlist[counter] =[shouldbe-1,leftx,lefty] ;rightlist[counter] =[shouldbe-1,rightx,righty]
          shouldbe = shouldbe + 1
 
        #for first entry only
        if counter == 0:
          leftlist[counter] =[shouldbe-1,leftx,lefty] ;rightlist[counter] =[shouldbe-1,rightx,righty]
          shouldbe = shouldbe + 1
 
      #start for discontinuous
      elif baselist[counter + 1][0] > shouldbe:
       leftlist[counter] = [shouldbe-1,leftx,lefty] ;rightlist[counter] =[shouldbe-1,rightx,righty]
       shouldbe = baselist[counter + 1][0] +1
 
      else:
        showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.makeLR') 
      counter = counter + 1
 
    #for last residue, print residue number
    if counter == (length-1):
      garbage = leftlist[counter]
      leftlist[counter] = [shouldbe-1,garbage[0],garbage[1]]
      garbage = rightlist[counter]
      rightlist[counter] = [shouldbe-1,garbage[0],garbage[1]]
 
    #change left/right list to dictionaries
    for item in leftlist:
      leftlut[item[0]] = item[1:]
    for item in rightlist:
      rightlut[item[0]] = item[1:]    
    return leftlut,rightlut

           
#################### various aspects of color #####################
    
   #general
   #calculates color for graph, changed to fit pyMol
   def floatRgb(self,mag, cmin, cmax):
      #normalize entries to 0-1
      try:
         # normalize to [0,1]
         x = float(mag-cmin)/float(cmax-cmin)
      except:
         # cmax = cmin
         x = 0.5	       
      
      #the easily changeable , rainbow method
      #these have the normalized value compared to B/distance (scale)
      #and the corresponding strength of the channel (intensity)
      #each color needs entries for 0 ->1 inclusive, in that order
      redscale=[0,0.15,0.5,0.7,1]
      redintensity=[0.7,0,0,1,1]
      greenscale=[0,0.2,0.4,0.6,1]
      greenintensity=[0,0,1,1,0]
      bluescale=[0,0.3,0.5,1]
      blueintensity=[1,1,0,0]
    
      loop=[0,1,2]
      list1=redscale
      list2=redintensity     
      #loop for each channel
      for count in loop:
         #enforce enqual number of entries
         if len(list1) != len(list2):
           showwarning(title='coloRNA Internal Error Check', 
               message='Unmatched list length in new color table. Go back to the code and fix it.') 
           return
         else: end = len(list1)
         
         c = int(1)
         tcolor = 'gar'         
         #check if mag = zero, each color needs a zero entry
         if list1[0] == x:
            tcolor=list2[0]
            c = end 
            
         #loop for all control points particular color
         while c < end:
            if list1[c] == x:
               tcolor=list2[c]
               break 
     
            elif x < list1[c]:               
               #interpolate, list1~x,list2~y, previous position=[c-1]
               tcolor= list2[c-1]+  (list2[c]-list2[c-1]) *( (x-list1[c-1]) /(list1[c]-list1[c-1]) )  
               break
            c = c + 1
            
         #check that someone gave new color
         if tcolor == 'gar':
            showwarning(title='coloRNA Internal Error Check', message='Error catch in self.floatRgb') 
            return
            
         #put tcolor to right color, switch list1&2 to next color
         if count == 0:
            red = tcolor
            list1=greenscale
            list2=greenintensity
         elif count == 1:
            green = tcolor
            list1=bluescale
            list2=blueintensity
         else:
            blue = tcolor

      """
      #oringal method modified for diff colors
      midpoint = float(0.3)
      gbegin = float(0.10)
      gend = float(0.95)
      #determine blue & red
      blue = min((max((4*(midpoint-x), 0.)), 1.))
      red  = min((max((4*(x-midpoint), 0.)), 1.))	
      #for green, this produces color gradient similar to Kakoli's/PyMol
      if x >=gbegin and x <= midpoint:
         green = (x-gbegin)*(1/(midpoint-gbegin))
      elif x > midpoint and x < gend:
	 green = (x-gend)*(1/(midpoint-gend))
      else:
	 green = 0	  
      #orignal
      #blue = min((max((4*(0.75-x), 0.)), 1.))
      #red  = min((max((4*(x-0.25), 0.)), 1.))
      #green= min((max((4*math.fabs(x-0.5)-1., 0.)), 1.))
      """
      return (red, green, blue)
	
   #general
   #calls floatRgb, returns certain type
   def strRgb(self,mag, cmin, cmax):
        #Return a tuple of strings to be used in Tk plots.
        red, green, blue = self.floatRgb(mag, cmin, cmax)       
        return "#%02x%02x%02x" % (red*255, green*255, blue*255) 
        
   #specific
   #determines color, but checks if using bcol or distance
   def runcolor(self,handle):
      #if using Bcolumn
      #if self.BcolLdGd.get() == 0:
      #handle -> residue ID
      tags = self.c.gettags(handle)
      rid=int(tags[0][5:])
      #residue ID to residue value
      if self.RCreslut.has_key(rid):
         value = self.Clut[rid]
         #if want min/max of colors set by thresholds
         if self.thrshctrlkey.get() == True:
             cvalue = self.strRgb(value,self.minthresh.get(),self.maxthresh.get()) 
         else:
             cvalue = self.strRgb(value,self.Cmin,self.Cmax) 
      return cvalue
      
##################### draw bonds ######################
     
   #general
   #graph all bonds
   def dspbonds(self, sticks, balls,xmov,ymov,lrgO=6,smlO=4,ciredge=1,color='black'):
      #delete all bonds
      self.c.delete('allbond')
      
      #graph bar bonds
      counter = 1      
      for entry in sticks:
        newentry=float(entry[0])+xmov,float(entry[1])+ymov,float(entry[2])+xmov,float(entry[3])+ymov
        self.c.create_line(newentry,width=1,fill=color,capstyle='round',
           tags=('barbond'+str(counter),'allbarbond','allbond')  )
        counter=counter+1
      
      #graph c(ircle)bonds
      counter=1
      for entry in balls:
         nametup=('cbond'+str(counter),'allcbond','allbond')
	 
	 #for filled circles
	 if entry[0] == int(3):
	    #determine if large/small
	    if entry[3] == float(2.5):
	       self.cbf(float(entry[1])+xmov,float(entry[2])+ymov,tig=nametup,lrgd=lrgO,clr=color)
	    else:
	       self.cbf(float(entry[1])+xmov,float(entry[2])+ymov,tig=nametup,large=False,lrgd=lrgO,smld=smlO,clr=color)
	       
	 #for hollow circles
	 elif entry[0] == int(2):
	    self.cbh(float(entry[1])+xmov,float(entry[2])+ymov,tig=nametup,lrgd=lrgO,thick=ciredge,clr=color)
	    
	 #wrongness
	 else:
	    showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.dspbonds') 
	 counter=counter+1
      return
               	   
   #general	
   #draw circle bond, filled
   def cbf(self,x,y,large=True,lrgd=6,smld=4,clr='black',tig=None):
     #if large, use lrgd, otherwise not
     if large == True:
       rad = lrgd/2
     else:
       rad = smld/2
     #calc bounding box
     x0=x-rad ; y0=y-rad
     x1=x+rad ; y1=y+rad
     #draw circle
     if tig != None:
       name = self.c.create_oval(x0,y0,x1,y1,fill=clr,outline=clr,tags=tig)
     else:
       name = self.c.create_oval(x0,y0,x1,y1,fill=clr,outline=clr)
       showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.cbf') 
     return name
   
   #general  
   #draw circle bond, hollow
   def cbh(self,x,y,lrgd=6,thick=1,clr='black',tig=None):
     #calc smaller boundingbox for larger line
     #while keeping outer diameter = cbf large
     rad = (lrgd+2 - (thick))/2 
     #calc bounding box
     x0=x-rad ; y0=y-rad
     x1=x+rad ; y1=y+rad
     if tig !=None:
       name=self.c.create_oval(x0,y0,x1,y1,fill="",outline=clr,width=thick,tags=tig)
     else:
       name=self.c.create_oval(x0,y0,x1,y1,fill="",outline=clr,width=thick)
       showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.cbh') 
     return name

####################### key creation ##########################

   #specific
   def runMK(self,cont,p3,p4,p5,p6,kvnum):
      #get top xy from preexisting key
      position = self.c.coords('keynum1')
      kind = self.c.type('keynum1')
      
      #found a key, use this for initial xy
      if len(position) > 0:
         if kind == 'polygon':
            #diamond
            if len(position) == int(8):
               p1= position[0] ; p2= position[3]
            #triangle
            elif len(position) == int(6):
               p1= position[4] ; p2= (position[1]+position[5]) /2
            else:
               showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.runMK') 
               
         elif kind == 'rectangle':
            p1= (position[0]+position[2]) /2 ; p2= (position[1]+position[3]) /2
         elif kind == 'oval':
            p1= (position[0]+position[2]) /2 ; p2= (position[1]+position[3]) /2
         else:
            showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.runMK Type 2') 
            
      #create defult x and y
      else:
         p1=int(300) ; p2=int(100)
         
     #default set key to current color max&min
      big = self.Cmax ; small = self.Cmin 
      #maybe set key min/max to currrent thresholds
      if self.thrshctrlkey.get() == True:
         big=self.maxthresh.get()
         small = self.minthresh.get()
         
      #call makekey
      if big != None or small != None:
         if cont == 1: self.makekey(p1,p2,p3,p4,p5,p6,big,small)
         self.makekvalue(p1,p2,p6,p4,kvnum,big,small)
      
   #general
   #destroy all key items, then recreate key shapes
   def makekey(self,x,y,num,space,shape,size,cmax,cmin ):
      #prep for iterative key creation
      self.c.delete('allkey')
      
      #key only make sense for at least two entries
      if int(num) > int(1):
         mult = int(num-1)
         valuestep = (cmax-cmin)/(num-1)
         value = (mult * valuestep) + cmin # = cmax
         newy=y
         counter=int(1)
      
         #make sure not infinte loop
         if valuestep > 0:
            #loop through each key entry, start from 'top' create each key symbol
            while value >= cmin:
               #list of tags
               names=('keynum'+str(counter),'allkshape','allkey')
               #get color
               cval = self.strRgb(value,cmin,cmax)
               #make shape
               self.shape(shape,x,newy,size,cval,names)
      
               #update stuff
               counter=counter+1
               newy=newy+space
               mult=mult-1
               value = (mult * valuestep) + cmin
      
   #general
   #destroy key value items, make new, if any key shapes present
   def makekvalue(self,fx,fy,shsz,space,vlen,cmax,cmin):
      #delete previous
      self.c.delete('allkvalue')
      #get num of shapes
      num = len(self.c.find_withtag('allkshape') )
      
      #if theres more than one shape, continue
      #calc values exactly same as key shape, but seperate
      if num > int(1) and vlen > 0:
         mult = int(num-1)
         valuestep = (cmax-cmin)/(num-1)
         value = (mult * valuestep) + cmin # = cmax
         newy=fy-1
         x=fx-shsz
         counter=int(1)
         
         #loop through key shapes
         while value >= cmin:
            #prep tags
            names= ('keyvnum'+str(counter),'allkvalue','allkey')
            svalue=str(value)
            #create
            self.c.create_text(x,newy,text=svalue[:vlen],anchor = E,
                               tags=names,font="Helvetica "+str(shsz) )
            
            #update
            counter=counter+1
            newy=newy+space
            mult=mult-1
            value = (mult * valuestep) + cmin
      
#################### shape files #########################

   #specific
   #this determines what shape 
   def shape(self,which,xs,ys,sz,tint,names):
      if which == int(0):
         name=self.scircle(xs,ys,sz,color=tint,tig=names)
      elif which == int(1):
         name=self.ssquare(xs,ys,sz,color=tint,tig=names)
      elif which == int(2):
         name=self.sdiamond(xs,ys,sz,color=tint,tig=names)
      elif which == int(3):
         name=self.striangle(xs,ys,sz,color=tint,tig=names,up=True)
      elif which == int(4):
         name=self.striangle(xs,ys,sz,color=tint,tig=names,up=False)
      else:
         showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.shape') 
      return name,which  
      
   #general  
   #square symbol
   def ssquare(self,x,y,size,color="black",tig=None):
     x0 = x - (size/2) ; y0 = y - (size/2)
     x1 = x + (size/2) ; y1 = y + (size/2)
     if tig != None:
       name = self.c.create_rectangle(x0,y0,x1,y1,fill=color,outline=color,tags=tig)
     else:
       name = self.c.create_rectangle(x0,y0,x1,y1,fill=color,outline=color)
       showwarning(title='coloRNA Internal Error Check', message='Error catch in self.ssquare') 
     return name
   
   #general  
   #circle symbol
   def scircle(self,x,y,size,color="black",tig=None):
     x0 = x - (size/2) ; y0 = y - (size/2)
     x1 = x + (size/2) ; y1 = y + (size/2)
     if tig != None:
       name = self.c.create_oval(x0,y0,x1,y1,fill=color,outline=color,tags=tig)
     else:
       name = self.c.create_oval(x0,y0,x1,y1,fill=color,outline=color)
       showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.scircle') 
     return name
   
   #general  
   #diamond symbol
   def sdiamond(self,x,y,size,color="black",tig=None): 
     #calc verticies
     x0=x ; x2=x ; y1=y ; y3=y
     x1=x+(size/2) ; x3=x-(size/2)
     y0=y-(size/2) ; y2=y+(size/2)
     zlist=(x0,y0,x1,y1,x2,y2,x3,y3)   
     if tig != None:
       name = self.c.create_polygon(zlist,fill=color,outline=color,splinesteps=0,tags=tig)
     else:
       name = self.c.create_polygon(zlist,fill=color,outline=color,splinesteps=0,)
       showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.sdiamond') 
     return name
   
   #general  
   #triangle symbol
   def striangle(self,x,y,size,color='black',up=True,tig=None):
     #seeing as only flipping up/down, x stays the same
     x0=x-(size/2) ; x1=x+(size/2) ; x2=x
     #flip or not
     if up == True:
       y0=y+(size/2) ; y1=y+(size/2) ; y2=y-(size/2)
     else:
       y0=y-(size/2) ; y1=y-(size/2) ; y2=y+(size/2)
     #shorten call and graph
     zlist=(x0,y0,x1,y1,x2,y2)
     if tig != None:
       name= self.c.create_polygon(zlist,fill=color,outline=color,splinesteps=0,tags=tig)
     else:
       name= self.c.create_polygon(zlist,fill=color,outline=color,splinesteps=0)
       showwarning(title='coloRNA Internal Error Check', message='ELSE catch in self.striangle') 
     return name

##################### read files #######################

   #general
   def readpolyfile(self,infile):
      #checks for inital lines to skip
      #reads multiple lines per realspace group
      #expects for each group:
      #   keyswitch,#,color
      #   x1
      #   y1
      #   xN
      #   yN
      #outputs list(entry)=[name,color,list[x],list[y]]

      f = open(infile,"r")  
      mainlist =[]              #list to collect info
      startreading = False      #switch to start using info
      keyswitch=['real','space','group'] #what to look for to start reading
      lettlist = string.letters #use lower/upper letters to name multi lines/group
      itercount = int(0)        #counter used to figure which lines
      lettcount = int(-1)       #counter for letters to add to grpnum
      nxtX = True               #shows if next line is x line, or yline=false
      linex = []
      liney = []
  
      #read every line of file
      for entry in f.readlines():
         #convert this line to words
         words = string.split(entry)
    
         #test to allow "header" space
         if startreading == False:
            if words[:3] == keyswitch:
               startreading = True
	
         #normal reading: found info and not empty line
         if startreading == True and len(words) > int(0):
      
            #for header lines
            if words[:3] == keyswitch:
               grpnum = words[3]       #base group number
	       grpcolor = words[4]     #color for whole group
	       lettcount = int(-1)     #amend lettlist
	       linex = [] ; liney = []
               nxtX = True
	
            #for x lines
            elif nxtX == True:
               #change words in string to float
               linex = map((lambda x:float(x)),words)
	       nxtX = False
	
            #for y lines, update mainlist, prep for multiple
            elif nxtX == False:
               #change words in string to float
               liney = map((lambda x:float(x)),words)
	
	       #if first/only line use base name
	       if lettcount < int(0):
	          name = grpnum
	       elif lettcount > (len(lettlist)-1):
	          #print "Too many lines/shapes for group ",grpnum 
                  showwarning(title='coloRNA Internal Error Check', message='Error catch in self.readpolyfile') 
	          return "Error, readpolyfile"
	       else:
	          name = grpnum + lettlist[lettcount]
	  
	       #"print" info to mainlist
	       mainlist.append([name,grpcolor,linex,liney])
	       #update for multi-lines
	       nxtX = True
	       lettcount = lettcount + 1
      f.close()	
      #warning window
      if len(mainlist) < 1:
         showwarning(title='Un-Expected File, No Data Read',
            message='You tried to use a file for colored groups that was not formated correctly.')
      return mainlist  
     
   #general
   #reads pdb file and stores certain info, from phosphates
   def readpdb(self,filename):
     #assume that filename has already been checked and is real
     #returns 1d list of residue numbers 
     #returns 1d list of data intensity numbers
     #can not use countercolumn or datacolumn after spliting because 
     #   of residues run into chain column, which run into residue name
    
     f = open(filename,"r")
     atom = 'ATOM'
     Bdict={}
     xyzdict={}
     numoutput =[]
     dataoutput =[]
     atomgoal = 'P'
     datamax = float("-1000.0")
     datamin = float("+2000.0")

     #print waringin to wait
     #print "\nConversion of .pdb file may take a minute, please wait."
     #read every line, and split into words, w
     for line in f.readlines():

        #if line is not header, and we want that data, ie phos.
        if line[0:4] == atom and line[13] == atomgoal:
	   #modified columns
	   countercolumn = line[22:26]
	   datacolumn = line[60:68]
           xpos = float(line[30:38])
           ypos = float(line[38:46])
           zpos = float(line[46:54])

	   #add residue number to list, and data value
           dataoutput.append(datacolumn) #w[datacolumn])
       
           #create dictionary for info
	   Bdict[int(countercolumn)] = float(datacolumn)
           xyzdict[int(countercolumn)] = [xpos,ypos,zpos]
	
     data=map((lambda x: float(x)),dataoutput)
     
     #error catch
     if len(data) > 0:
        datamin=min(data)
        datamax=max(data)
	
     #warning
     f.close()
     #warning window
     if len(Bdict) < 1 or len(xyzdict) < 1:
        showwarning(title='Un-Expected File, No Data Read',
          message='You tried to use a file for PDB that was not formated correctly.') 
     return  Bdict, datamin, datamax, xyzdict
     
   #general  
   #reads output of read2dps.py, looks for *search flags  
   def readmodps(self,filename):
    #assume filename is kosher    
    #outputs baselist = 1d list of list(residue counter,letter,x,y)
    #outputs barbond = 1d list of list(x1 y1 x2 y2)
    #outputs circlebond = 1d list of list(type int, x, y, diameter)
    	      
    #set flags and what to search for
    bondsearch=['bond','info','start','here']   
    basesearch=['base','info','start','here']
    bondflag=False
    baseflag=False
    
    #basic begining
    barbond=[]
    circlebond=[]
    fcirclebondbig=[]
    fcirclebondsmall=[]
    baselist=[]
    f = open(filename,"r")
    
    #keep track of xyminmax for invisible line
    xmin=ymin=float("+2000.0")
    xmax=ymax=float("-1000.0")
    
    #read every line, and split into words, w
    for line in f.readlines():
      w=string.split(line)
      #set bond flag
      if w==bondsearch: 
         bondflag = True
	 continue
      #set base flag, unset bond flag
      if w==basesearch: 
         bondflag = False
	 baseflag = True
	 continue
       
      #if bonds, create either round/bar list
      if bondflag == True:
         #if bond is a bar bond
	 if int(w[0]) == int(1.0):
	    barbond.append(w[1:])
	 #if bond is a circle
	 else :
	    #convert strings to floats
	    b=map(lambda x:float(x),w)
	    circlebond.append(b)
	 continue
	      
      #if bases, write to base list
      if baseflag == True:
         #add to base list
	 addon = [int(w[0]), str(w[1]), float(w[2]),float(w[3])]
	 baselist.append(addon)
	 
	 #keep track of min,max for xy
	 if float(w[2]) < xmin: xmin = float(w[2])
	 if float(w[3]) < ymin: ymin = float(w[3])
	 if float(w[2]) > xmax: xmax = float(w[2])
	 if float(w[3]) > ymax: ymax = float(w[3])
	 continue
    f.close()
    #warning window 
    if len(baselist) < 1:
        showwarning(title='Un-Expected File, No Data Read',
          message='You tried to use an XY data file that was not formated correctly.') 
    return baselist,barbond,circlebond 
   
   #general
   def readtnt(self,filename):
      #every list entry is a list of related residues
      #from grep " 0 " > output.file
      f=open(filename,"r")
      bar = "|"
      regionlist =[]
      currentlist = []
      istherenum = int(0)
      isthereR = int(0)
 
      #read every line, split to phrase by bar
      for line in f.readlines():
         #check
         if bar in line:
            p=string.split(line,bar)
            #read every phrase
            for phrase in p:
 
               #does R-factor appear?
               isthereR = string.find(phrase,"R-factor")
               #throw out "typical" first phrase
               if (isthereR < 0):

                  #split phrase into words
                  w=string.split(phrase)
                  #throw out continuing phrase
                  if (len(w) > 1) and len(w) < 6:
 
                     #add residues to current group
                     currentlist=currentlist + range(int(w[0]),(int(w[2])+1))
 
                     #is this last entry in group
                     istherenum = string.find(phrase,r"#")
                     #if is last phrase,this is where 2 lines are read
                     if (istherenum > 0):
                        regionlist.append(currentlist)
                        currentlist = []
      f.close()
      if len(regionlist) < 1:
        showwarning(title='Un-Expected File, No Data Read',
                 message='You tried to use a TNT file that was not formated correctly.') 
      return regionlist 

##when started by itself, and not used as a class, this starts program
if __name__ == '__main__':     
   root=Tk()
   root.title("coloRNA")
   root.geometry('+%d+%d' % (25,25))
   go = COLORNA(root)
   root.mainloop()
